#!/usr/bin/python

# Copyright (c) 2021, quidame <quidame@poivron.org>
# Copyright (c) 2013, Alexander Bulimov <lazywolf0@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Alexander Bulimov (@abulimov)
  - quidame (@quidame)
module: filesystem
short_description: Makes a filesystem
description:
  - This module creates a filesystem.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - If O(state=present), the filesystem is created if it does not already exist, that is the default behaviour if O(state)
        is omitted.
      - If O(state=absent), filesystem signatures on O(dev) are wiped if it contains a filesystem (as known by C(blkid)).
      - When O(state=absent), all other options but O(dev) are ignored, and the module does not fail if the device O(dev)
        does not actually exist.
    type: str
    choices: [present, absent]
    default: present
    version_added: 1.3.0
  fstype:
    choices: [bcachefs, btrfs, ext2, ext3, ext4, ext4dev, f2fs, lvm, ocfs2, reiserfs, xfs, vfat, swap, ufs, gfs2]
    description:
      - Filesystem type to be created. This option is required with O(state=present) (or if O(state) is omitted).
      - V(ufs) support has been added in community.general 3.4.0.
      - V(bcachefs) support has been added in community.general 8.6.0.
      - V(gfs2) support has been added in community.general 13.1.0.
    type: str
    aliases: [type]
  dev:
    description:
      - Target path to block device (Linux) or character device (FreeBSD) or regular file (both).
      - When setting Linux-specific filesystem types on FreeBSD, this module only works when applying to regular files, also known as
        disk images.
      - Currently V(lvm) (Linux-only) and V(ufs) (FreeBSD-only) do not support a regular file as their target O(dev).
      - Support for character devices on FreeBSD has been added in community.general 3.4.0.
    type: path
    required: true
    aliases: [device]
  force:
    description:
      - If V(true), allows to create new filesystem on devices that already has filesystem.
    type: bool
    default: false
  resizefs:
    description:
      - If V(true), if the block device and filesystem size differ, grow the filesystem into the space.
      - >-
        Supported when O(fstype) is one of: V(bcachefs), V(btrfs), V(ext2), V(ext3), V(ext4), V(ext4dev), V(f2fs), V(lvm), V(xfs), V(ufs) and V(vfat).
        Attempts to resize other filesystem types fail.
      - XFS only grows if mounted. Currently, the module is based on commands from C(util-linux) package to perform operations,
        so resizing of XFS is not supported on FreeBSD systems.
      - VFAT is likely to fail if C(fatresize < 1.04).
      - Mutually exclusive with O(uuid).
    type: bool
    default: false
  opts:
    description:
      - List of options to be passed to C(mkfs) command.
    type: str
  uuid:
    description:
      - Set filesystem's UUID to the given value.
      - The UUID options specified in O(opts) take precedence over this value.
      - See xfs_admin(8) (C(xfs)), tune2fs(8) (C(ext2), C(ext3), C(ext4), C(ext4dev)) for possible values.
      - For O(fstype=lvm) the value is ignored, it resets the PV UUID if set.
      - Supported for O(fstype) being one of V(bcachefs), V(ext2), V(ext3), V(ext4), V(ext4dev), V(lvm), V(gfs2) or V(xfs).
      - This is B(not idempotent). Specifying this option always results in a change.
      - Mutually exclusive with O(resizefs).
    type: str
    version_added: 7.1.0
  label:
    description:
      - Set filesystem's label to the given value.
      - The label options specified in O(opts) take precedence over this value.
      - Supported for O(fstype) being one of V(gfs2). Support for other filesystems might be added later.
      - On GFS2 this sets the lock table with the C(-t) option of mkfs.gfs2(8). It has to be in the form V(CLUSTERNAME:LOCKSPACE).
    type: str
    version_added: 13.1.0
requirements:
  - Uses specific tools related to the O(fstype) for creating or resizing a filesystem (from packages e2fsprogs, xfsprogs,
    dosfstools, and so on).
  - Uses generic tools mostly related to the Operating System (Linux or FreeBSD) or available on both, as C(blkid).
  - On FreeBSD, either C(util-linux) or C(e2fsprogs) package is required.
notes:
  - Potential filesystems on O(dev) are checked using C(blkid). In case C(blkid) is unable to detect a filesystem (and in
    case C(fstyp) on FreeBSD is also unable to detect a filesystem), this filesystem is overwritten even if O(force=false).
  - On FreeBSD systems, both C(e2fsprogs) and C(util-linux) packages provide a C(blkid) command that is compatible with this
    module. However, these packages conflict with each other, and only the C(util-linux) package provides the command required
    to not fail when O(state=absent).
seealso:
  - module: community.general.filesize
  - module: ansible.posix.mount
  - name: xfs_admin(8) manpage for Linux
    description: Manual page of the GNU/Linux's xfs_admin implementation.
    link: https://man7.org/linux/man-pages/man8/xfs_admin.8.html
  - name: tune2fs(8) manpage for Linux
    description: Manual page of the GNU/Linux's tune2fs implementation.
    link: https://man7.org/linux/man-pages/man8/tune2fs.8.html
"""

EXAMPLES = r"""
- name: Create a ext2 filesystem on /dev/sdb1
  community.general.filesystem:
    fstype: ext2
    dev: /dev/sdb1

- name: Create a ext4 filesystem on /dev/sdb1 and check disk blocks
  community.general.filesystem:
    fstype: ext4
    dev: /dev/sdb1
    opts: -cc

- name: Blank filesystem signature on /dev/sdb1
  community.general.filesystem:
    dev: /dev/sdb1
    state: absent

- name: Create a filesystem on top of a regular file
  community.general.filesystem:
    dev: /path/to/disk.img
    fstype: vfat

- name: Reset an xfs filesystem UUID on /dev/sdb1
  community.general.filesystem:
    fstype: xfs
    dev: /dev/sdb1
    uuid: generate

- name: Reset an ext4 filesystem UUID on /dev/sdb1
  community.general.filesystem:
    fstype: ext4
    dev: /dev/sdb1
    uuid: random

- name: Reset an LVM filesystem (PV) UUID on /dev/sdc
  community.general.filesystem:
    fstype: lvm
    dev: /dev/sdc
    uuid: random
"""

import os
import platform
import re
import stat

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lvm import pvs_runner
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class Device:
    def __init__(self, module, path):
        self.module = module
        self.path = path

    def size(self):
        """Return size in bytes of device. Returns int"""
        statinfo = os.stat(self.path)
        if stat.S_ISBLK(statinfo.st_mode):
            blockdev_cmd = self.module.get_bin_path("blockdev", required=True)
            dummy, out, dummy = self.module.run_command([blockdev_cmd, "--getsize64", self.path], check_rc=True)
            devsize_in_bytes = int(out)
        elif stat.S_ISCHR(statinfo.st_mode) and platform.system() == "FreeBSD":
            diskinfo_cmd = self.module.get_bin_path("diskinfo", required=True)
            dummy, out, dummy = self.module.run_command([diskinfo_cmd, self.path], check_rc=True)
            devsize_in_bytes = int(out.split()[2])
        elif os.path.isfile(self.path):
            devsize_in_bytes = os.path.getsize(self.path)
        else:
            self.module.fail_json(changed=False, msg=f"Target device not supported: {self}")

        return devsize_in_bytes

    def get_mountpoint(self):
        """Return (first) mountpoint of device. Returns None when not mounted."""
        cmd_findmnt = self.module.get_bin_path("findmnt", required=True)

        # find mountpoint
        rc, mountpoint, dummy = self.module.run_command(
            [cmd_findmnt, "--mtab", "--noheadings", "--output", "TARGET", "--source", self.path], check_rc=False
        )
        if rc != 0:
            mountpoint = None
        else:
            mountpoint = mountpoint.split("\n")[0]

        return mountpoint

    def __str__(self):
        return self.path


class Filesystem:
    MKFS: str | None = None
    MKFS_FORCE_FLAGS: list[str] | None = []
    MKFS_SET_UUID_OPTIONS: list[str] | None = None
    MKFS_SET_UUID_EXTRA_OPTIONS: list[str] | None = []
    MKFS_SET_LABEL_OPTIONS: list[str] | None = None
    INFO: str | None = None
    GROW: str | None = None
    GROW_SLACK: int = 0
    GROW_MAX_SPACE_FLAGS: list[str] | None = []
    GROW_MOUNTPOINT_ONLY = False
    CHANGE_UUID: str | None = None
    CHANGE_UUID_OPTION: str | None = None
    CHANGE_UUID_OPTION_HAS_ARG = True

    LANG_ENV = {"LANGUAGE": "C", "LC_ALL": "C"}

    def __init__(self, module):
        self.module = module

    @property
    def fstype(self):
        return type(self).__name__

    def get_fs_size(self, dev):
        """Return size in bytes of filesystem on device (integer).
        Should query the info with a per-fstype command that can access the
        device whenever it is mounted or not, and parse the command output.
        Parser must ensure to return an integer, or raise a ValueError.
        """
        raise NotImplementedError()

    def create(self, opts, dev, uuid=None, label=None):
        if self.module.check_mode:
            return

        if uuid and self.MKFS_SET_UUID_OPTIONS:
            if not (set(self.MKFS_SET_UUID_OPTIONS) & set(opts)):
                opts += [self.MKFS_SET_UUID_OPTIONS[0], uuid] + self.MKFS_SET_UUID_EXTRA_OPTIONS
        if label is not None and self.MKFS_SET_LABEL_OPTIONS:
            if not (set(self.MKFS_SET_LABEL_OPTIONS) & set(opts)):
                opts += [self.MKFS_SET_LABEL_OPTIONS[0], label]

        mkfs = self.module.get_bin_path(self.MKFS, required=True)
        cmd = [mkfs] + self.MKFS_FORCE_FLAGS + opts + [str(dev)]
        self.module.run_command(cmd, check_rc=True)
        if uuid and self.CHANGE_UUID and self.MKFS_SET_UUID_OPTIONS is None:
            self.change_uuid(new_uuid=uuid, dev=dev)

    def wipefs(self, dev):
        if self.module.check_mode:
            return

        # wipefs comes with util-linux package (as 'blockdev' & 'findmnt' above)
        # that is ported to FreeBSD. The use of dd as a portable fallback is
        # not doable here if it needs get_mountpoint() (to prevent corruption of
        # a mounted filesystem), since 'findmnt' is not available on FreeBSD,
        # even in util-linux port for this OS.
        wipefs = self.module.get_bin_path("wipefs", required=True)
        cmd = [wipefs, "--all", str(dev)]
        self.module.run_command(cmd, check_rc=True)

    def grow_cmd(self, target):
        """Build and return the resizefs commandline as list."""
        cmdline = [self.module.get_bin_path(self.GROW, required=True)]
        cmdline += self.GROW_MAX_SPACE_FLAGS + [target]
        return cmdline

    def grow(self, dev):
        """Get dev and fs size and compare. Returns stdout of used command."""
        devsize_in_bytes = dev.size()

        try:
            fssize_in_bytes = self.get_fs_size(dev)
        except NotImplementedError:
            self.module.fail_json(msg=f"module does not support resizing {self.fstype} filesystem yet")
        except ValueError as err:
            self.module.warn(f"unable to process {self.INFO} output '{err}'")
            self.module.fail_json(msg=f"unable to process {self.INFO} output for {dev}")

        if fssize_in_bytes + self.GROW_SLACK >= devsize_in_bytes:
            self.module.exit_json(changed=False, msg=f"{self.fstype} filesystem is using the whole device {dev}")
        elif self.module.check_mode:
            self.module.exit_json(changed=True, msg=f"resizing filesystem {self.fstype} on device {dev}")

        if self.GROW_MOUNTPOINT_ONLY:
            mountpoint = dev.get_mountpoint()
            if not mountpoint:
                self.module.fail_json(msg=f"{dev} needs to be mounted for {self.fstype} operations")
            grow_target = mountpoint
        else:
            grow_target = str(dev)

        dummy, out, dummy = self.module.run_command(self.grow_cmd(grow_target), check_rc=True)
        return out

    def change_uuid_cmd(self, new_uuid, target):
        """Build and return the UUID change command line as list."""
        cmdline = [self.module.get_bin_path(self.CHANGE_UUID, required=True)]
        if self.CHANGE_UUID_OPTION_HAS_ARG:
            cmdline += [self.CHANGE_UUID_OPTION, new_uuid, target]
        else:
            cmdline += [self.CHANGE_UUID_OPTION, target]
        return cmdline

    def change_uuid(self, new_uuid, dev):
        """Change filesystem UUID. Returns stdout of used command"""
        if self.module.check_mode:
            self.module.exit_json(change=True, msg=f"Changing {self.fstype} filesystem UUID on device {dev}")

        dummy, out, dummy = self.module.run_command(
            self.change_uuid_cmd(new_uuid=new_uuid, target=str(dev)), check_rc=True
        )
        return out


class Ext(Filesystem):
    MKFS_FORCE_FLAGS = ["-F"]
    MKFS_SET_UUID_OPTIONS = ["-U"]
    INFO = "tune2fs"
    GROW = "resize2fs"
    CHANGE_UUID = "tune2fs"
    CHANGE_UUID_OPTION = "-U"

    def get_fs_size(self, dev):
        """Get Block count and Block size and return their product."""
        cmd = self.module.get_bin_path(self.INFO, required=True)
        dummy, out, dummy = self.module.run_command([cmd, "-l", str(dev)], check_rc=True, environ_update=self.LANG_ENV)

        block_count = block_size = None
        for line in out.splitlines():
            if "Block count:" in line:
                block_count = int(line.split(":")[1].strip())
            elif "Block size:" in line:
                block_size = int(line.split(":")[1].strip())
            if None not in (block_size, block_count):
                break
        else:
            raise ValueError(repr(out))

        return block_size * block_count


class Ext2(Ext):
    MKFS = "mkfs.ext2"


class Ext3(Ext):
    MKFS = "mkfs.ext3"


class Ext4(Ext):
    MKFS = "mkfs.ext4"


class XFS(Filesystem):
    MKFS = "mkfs.xfs"
    MKFS_FORCE_FLAGS = ["-f"]
    INFO = "xfs_info"
    GROW = "xfs_growfs"
    # XFS (defaults with 4KiB blocksize) requires at least 64 block of free
    # space to add a new allocation group, avoid resizing (noop, but shown as
    # diff) if the difference between the filesystem and the device is less
    GROW_SLACK = 64 * 4096 - 1
    GROW_MOUNTPOINT_ONLY = True
    CHANGE_UUID = "xfs_admin"
    CHANGE_UUID_OPTION = "-U"

    def get_fs_size(self, dev):
        """Get bsize and blocks and return their product."""
        cmdline = [self.module.get_bin_path(self.INFO, required=True)]

        # Depending on the versions, xfs_info is able to get info from the
        # device, whenever it is mounted or not, or only if unmounted, or
        # only if mounted, or not at all. For any version until now, it is
        # able to query info from the mountpoint. So try it first, and use
        # device as the last resort: it may or may not work.
        mountpoint = dev.get_mountpoint()
        if mountpoint:
            cmdline += [mountpoint]
        else:
            cmdline += [str(dev)]
        dummy, out, dummy = self.module.run_command(cmdline, check_rc=True, environ_update=self.LANG_ENV)

        block_size = block_count = None
        for line in out.splitlines():
            col = line.split("=")
            if col[0].strip() == "data":
                if col[1].strip() == "bsize":
                    block_size = int(col[2].split()[0])
                if col[2].split()[1] == "blocks":
                    block_count = int(col[3].split(",")[0])
            if None not in (block_size, block_count):
                break
        else:
            raise ValueError(repr(out))

        return block_size * block_count


class Reiserfs(Filesystem):
    MKFS = "mkfs.reiserfs"
    MKFS_FORCE_FLAGS = ["-q"]


class Bcachefs(Filesystem):
    MKFS = "mkfs.bcachefs"
    MKFS_FORCE_FLAGS = ["--force"]
    MKFS_SET_UUID_OPTIONS = ["-U", "--uuid"]
    INFO = "bcachefs"
    GROW = "bcachefs"
    GROW_MAX_SPACE_FLAGS = ["device", "resize"]

    def get_fs_size(self, dev):
        """Return size in bytes of filesystem on device (integer)."""
        dummy, stdout, dummy = self.module.run_command(
            [self.module.get_bin_path(self.INFO), "show-super", str(dev)], check_rc=True
        )

        for line in stdout.splitlines():
            if "Size: " in line:
                parts = line.split()
                unit = parts[2]

                base = None
                exp = None

                units_2 = ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"]
                units_10 = ["B", "kB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"]

                try:
                    exp = units_2.index(unit)
                    base = 1024
                except ValueError:
                    exp = units_10.index(unit)
                    base = 1000

                if exp == 0:
                    value = int(parts[1])
                else:
                    value = float(parts[1])

                if base is not None and exp is not None:
                    return int(value * pow(base, exp))

        raise ValueError(repr(stdout))


class Btrfs(Filesystem):
    MKFS = "mkfs.btrfs"
    INFO = "btrfs"
    GROW = "btrfs"
    GROW_MAX_SPACE_FLAGS = ["filesystem", "resize", "max"]
    GROW_MOUNTPOINT_ONLY = True

    def __init__(self, module):
        super().__init__(module)
        mkfs = self.module.get_bin_path(self.MKFS, required=True)
        dummy, stdout, stderr = self.module.run_command([mkfs, "--version"], check_rc=True)
        match = re.search(r" v([0-9.]+)", stdout)
        if not match:
            # v0.20-rc1 use stderr
            match = re.search(r" v([0-9.]+)", stderr)
        if match:
            # v0.20-rc1 doesn't have --force parameter added in following version v3.12
            if LooseVersion(match.group(1)) >= LooseVersion("3.12"):
                self.MKFS_FORCE_FLAGS = ["-f"]
        else:
            # assume version is greater or equal to 3.12
            self.MKFS_FORCE_FLAGS = ["-f"]
            self.module.warn(f"Unable to identify mkfs.btrfs version ({stdout!r}, {stderr!r})")

    def get_fs_size(self, dev):
        """Return size in bytes of filesystem on device (integer)."""
        mountpoint = dev.get_mountpoint()
        if not mountpoint:
            self.module.fail_json(msg=f"{dev} needs to be mounted for {self.fstype} operations")

        dummy, stdout, dummy = self.module.run_command(
            [self.module.get_bin_path(self.INFO), "filesystem", "usage", "-b", mountpoint], check_rc=True
        )
        for line in stdout.splitlines():
            if "Device size" in line:
                return int(line.split()[-1])
        raise ValueError(repr(stdout))


class Ocfs2(Filesystem):
    MKFS = "mkfs.ocfs2"
    MKFS_FORCE_FLAGS = ["-Fx"]


class F2fs(Filesystem):
    MKFS = "mkfs.f2fs"
    INFO = "dump.f2fs"
    GROW = "resize.f2fs"

    def __init__(self, module):
        super().__init__(module)
        mkfs = self.module.get_bin_path(self.MKFS, required=True)
        dummy, out, dummy = self.module.run_command([mkfs, os.devnull], check_rc=False, environ_update=self.LANG_ENV)
        # Looking for "	F2FS-tools: mkfs.f2fs Ver: 1.10.0 (2018-01-30)"
        # mkfs.f2fs displays version since v1.2.0
        match = re.search(r"F2FS-tools: mkfs.f2fs Ver: ([0-9.]+) \(", out)
        if match is not None:
            # Since 1.9.0, mkfs.f2fs check overwrite before make filesystem
            # before that version -f switch wasn't used
            if LooseVersion(match.group(1)) >= LooseVersion("1.9.0"):
                self.MKFS_FORCE_FLAGS = ["-f"]

    def get_fs_size(self, dev):
        """Get sector size and total FS sectors and return their product."""
        cmd = self.module.get_bin_path(self.INFO, required=True)
        dummy, out, dummy = self.module.run_command([cmd, str(dev)], check_rc=True, environ_update=self.LANG_ENV)
        sector_size = sector_count = None
        for line in out.splitlines():
            if "Info: sector size = " in line:
                # expected: 'Info: sector size = 512'
                sector_size = int(line.split()[4])
            elif "Info: total FS sectors = " in line:
                # expected: 'Info: total FS sectors = 102400 (50 MB)'
                sector_count = int(line.split()[5])
            if None not in (sector_size, sector_count):
                break
        else:
            raise ValueError(repr(out))

        return sector_size * sector_count


class VFAT(Filesystem):
    INFO = "fatresize"
    GROW = "fatresize"
    GROW_MAX_SPACE_FLAGS = ["-s", "max"]

    def __init__(self, module):
        super().__init__(module)
        if platform.system() == "FreeBSD":
            self.MKFS = "newfs_msdos"
        else:
            self.MKFS = "mkfs.vfat"

    def get_fs_size(self, dev):
        """Get and return size of filesystem, in bytes."""
        cmd = self.module.get_bin_path(self.INFO, required=True)
        dummy, out, dummy = self.module.run_command(
            [cmd, "--info", str(dev)], check_rc=True, environ_update=self.LANG_ENV
        )
        fssize = None
        for line in out.splitlines()[1:]:
            parts = line.split(":", 1)
            if len(parts) < 2:
                continue
            param, value = parts
            if param.strip() in ("Size", "Cur size"):
                fssize = int(value.strip())
                break
        else:
            raise ValueError(repr(out))

        return fssize


class LVM(Filesystem):
    MKFS = "pvcreate"
    MKFS_FORCE_FLAGS = ["-f"]
    MKFS_SET_UUID_OPTIONS = ["-u", "--uuid"]
    MKFS_SET_UUID_EXTRA_OPTIONS = ["--norestorefile"]
    INFO = "pvs"
    GROW = "pvresize"
    CHANGE_UUID = "pvchange"
    CHANGE_UUID_OPTION = "-u"
    CHANGE_UUID_OPTION_HAS_ARG = False

    def __init__(self, module):
        super().__init__(module)
        self._pvs = pvs_runner(module)

    def get_fs_size(self, dev):
        """Get and return PV size, in bytes."""
        with self._pvs("noheadings nosuffix units fields devices") as ctx:
            dummy, size, dummy = ctx.run(units="b", fields="pv_size", devices=[str(dev)])
        return int(size.strip())


class Swap(Filesystem):
    MKFS = "mkswap"
    MKFS_FORCE_FLAGS = ["-f"]


class UFS(Filesystem):
    MKFS = "newfs"
    INFO = "dumpfs"
    GROW = "growfs"
    GROW_MAX_SPACE_FLAGS = ["-y"]

    def get_fs_size(self, dev):
        """Get providersize and fragment size and return their product."""
        cmd = self.module.get_bin_path(self.INFO, required=True)
        dummy, out, dummy = self.module.run_command([cmd, str(dev)], check_rc=True, environ_update=self.LANG_ENV)

        fragmentsize = providersize = None
        for line in out.splitlines():
            if line.startswith("fsize"):
                fragmentsize = int(line.split()[1])
            elif "providersize" in line:
                providersize = int(line.split()[-1])
            if None not in (fragmentsize, providersize):
                break
        else:
            raise ValueError(repr(out))

        return fragmentsize * providersize


class GFS2(Filesystem):
    MKFS = "mkfs.gfs2"
    MKFS_FORCE_FLAGS = ["-O"]
    MKFS_SET_UUID_OPTIONS = ["-U"]
    MKFS_SET_LABEL_OPTIONS = ["-t"]


FILESYSTEMS = {
    "bcachefs": Bcachefs,
    "ext2": Ext2,
    "ext3": Ext3,
    "ext4": Ext4,
    "ext4dev": Ext4,
    "f2fs": F2fs,
    "reiserfs": Reiserfs,
    "xfs": XFS,
    "btrfs": Btrfs,
    "vfat": VFAT,
    "ocfs2": Ocfs2,
    "LVM2_member": LVM,
    "swap": Swap,
    "ufs": UFS,
    "gfs2": GFS2,
}


def main():
    friendly_names = {
        "lvm": "LVM2_member",
    }

    fstypes = set(FILESYSTEMS.keys()) - set(friendly_names.values()) | set(friendly_names.keys())

    # There is no "single command" to manipulate filesystems, so we map them all out and their options
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(type="str", default="present", choices=["present", "absent"]),
            fstype=dict(type="str", aliases=["type"], choices=list(fstypes)),
            dev=dict(type="path", required=True, aliases=["device"]),
            opts=dict(type="str"),
            force=dict(type="bool", default=False),
            resizefs=dict(type="bool", default=False),
            uuid=dict(type="str"),
            label=dict(type="str"),
        ),
        required_if=[("state", "present", ["fstype"])],
        mutually_exclusive=[
            ("resizefs", "uuid"),
        ],
        supports_check_mode=True,
    )

    state = module.params["state"]
    dev = module.params["dev"]
    fstype = module.params["fstype"]
    opts = module.params["opts"]
    force = module.params["force"]
    resizefs = module.params["resizefs"]
    uuid = module.params["uuid"]
    label = module.params["label"]

    mkfs_opts = []
    if opts is not None:
        mkfs_opts = opts.split()

    changed = False

    if not os.path.exists(dev):
        msg = f"Device {dev} not found."
        if state == "present":
            module.fail_json(msg=msg)
        else:
            module.exit_json(msg=msg)

    dev = Device(module, dev)

    # In case blkid/fstyp isn't able to identify an existing filesystem, device
    # is considered as empty, then this existing filesystem would be overwritten
    # even if force isn't enabled.
    cmd = module.get_bin_path("blkid", required=True)
    rc, raw_fs, err = module.run_command([cmd, "-c", os.devnull, "-o", "value", "-s", "TYPE", str(dev)])
    fs = raw_fs.strip()
    # BusyBox blkid may ignore -o/-s flags and output the full device info line,
    # e.g. '/dev/sda2: UUID="..." TYPE="btrfs"' instead of just 'btrfs'
    if fs and " " in fs:
        type_match = re.search(r'\bTYPE="?([^"\s]+)"?', raw_fs)
        fs = type_match.group(1) if type_match else ""
    if not fs and platform.system() == "FreeBSD":
        cmd = module.get_bin_path("fstyp", required=True)
        rc, raw_fs, err = module.run_command([cmd, str(dev)])
        fs = raw_fs.strip()

    if state == "present":
        if fstype in friendly_names:
            fstype = friendly_names[fstype]

        try:
            klass = FILESYSTEMS[fstype]
        except KeyError:
            module.fail_json(changed=False, msg=f"module does not support this filesystem ({fstype}) yet.")

        filesystem = klass(module)

        if uuid and not (filesystem.CHANGE_UUID or filesystem.MKFS_SET_UUID_OPTIONS):
            module.fail_json(
                changed=False, msg=f"module does not support UUID option for this filesystem ({fstype}) yet."
            )

        same_fs = fs and FILESYSTEMS.get(fs) == FILESYSTEMS[fstype]
        if same_fs and not resizefs and not uuid and not force:
            module.exit_json(changed=False)
        elif same_fs:
            if resizefs:
                if not filesystem.GROW:
                    module.fail_json(changed=False, msg=f"module does not support resizing {fstype} filesystem yet.")

                out = filesystem.grow(dev)

                module.exit_json(changed=True, msg=out)
            elif uuid:
                out = filesystem.change_uuid(new_uuid=uuid, dev=dev)

                module.exit_json(changed=True, msg=out)
        elif fs and not force:
            module.fail_json(msg=f"'{dev}' is already used as {fs}, use force=true to overwrite", rc=rc, err=err)

        # create fs
        filesystem.create(opts=mkfs_opts, dev=dev, uuid=uuid, label=label)
        changed = True

    elif fs:
        # wipe fs signatures
        filesystem = Filesystem(module)
        filesystem.wipefs(dev)
        changed = True

    module.exit_json(changed=changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_policy_info
version_added: 5.0.0
short_description: Retrieve inline IAM policies for users, groups, and roles
description:
  - Supports fetching of inline IAM policies for IAM users, groups and roles.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  iam_type:
    description:
      - Type of IAM resource you wish to retrieve inline policies for.
    required: true
    choices: [ "user", "group", "role"]
    type: str
  iam_name:
    description:
      - Name of IAM resource you wish to retrieve inline policies for. In other words, the user name, group name or role name.
    required: true
    type: str
  policy_name:
    description:
      - Name of a specific IAM inline policy you with to retrieve.
    required: false
    type: str
author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Describe all inline IAM policies on an IAM User
  amazon.aws.iam_policy_info:
    iam_type: user
    iam_name: example_user

- name: Describe a specific inline policy on an IAM Role
  amazon.aws.iam_policy_info:
    iam_type: role
    iam_name: example_role
    policy_name: example_policy
"""

RETURN = r"""
policies:
    description: A list containing the matching IAM inline policy names and their data.
    returned: success
    type: list
    elements: dict
    contains:
        policy_name:
            description: The Name of the inline policy.
            returned: success
            type: str
            sample: "test-ec2-list-policy"
        policy_document:
            description: The JSON document representing the inline IAM policy.
            returned: success
            type: str
            sample: {
                        "Statement": [
                            {
                                "Action": "ec2:DescribeAccountAttributes",
                                "Effect": "Allow",
                                "Resource": "*",
                                "Sid": "VisualEditor0"
                            }
                        ],
                        "Version": "2012-10-17"
                    }
policy_names:
    description: A list of matching names of the IAM inline policies on the queried object.
    returned: success
    type: list
    sample: [
                "example-ec2-policy-1",
                "example-ec2-policy-2"
            ]
all_policy_names:
    description: A list of names of all of the IAM inline policies on the queried object.
    returned: success
    type: list
    sample: [
                "example-ec2-policy-1",
                "example-ec2-policy-2"
            ]
"""

try:
    import botocore
except ImportError:
    pass

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


class Policy:
    def __init__(self, client, name, policy_name):
        self.client = client
        self.name = name
        self.policy_name = policy_name
        self.changed = False

    @staticmethod
    def _iam_type():
        return ""

    def _list(self, name):
        return {}

    def list(self):
        return self._list(self.name).get("PolicyNames", [])

    def _get(self, name, policy_name):
        return "{}"

    def get(self, policy_name):
        return self._get(self.name, policy_name)["PolicyDocument"]

    def get_all(self):
        policies = list()
        for policy in self.list():
            policies.append({"policy_name": policy, "policy_document": self.get(policy)})
        return policies

    def run(self):
        policy_list = self.list()
        ret_val = {"changed": False, self._iam_type() + "_name": self.name, "all_policy_names": policy_list}
        if self.policy_name is None:
            ret_val.update(policies=self.get_all())
            ret_val.update(policy_names=policy_list)
        elif self.policy_name in policy_list:
            ret_val.update(policies=[{"policy_name": self.policy_name, "policy_document": self.get(self.policy_name)}])
            ret_val.update(policy_names=[self.policy_name])
        return ret_val


class UserPolicy(Policy):
    @staticmethod
    def _iam_type():
        return "user"

    def _list(self, name):
        return self.client.list_user_policies(aws_retry=True, UserName=name)

    def _get(self, name, policy_name):
        return self.client.get_user_policy(aws_retry=True, UserName=name, PolicyName=policy_name)


class RolePolicy(Policy):
    @staticmethod
    def _iam_type():
        return "role"

    def _list(self, name):
        return self.client.list_role_policies(aws_retry=True, RoleName=name)

    def _get(self, name, policy_name):
        return self.client.get_role_policy(aws_retry=True, RoleName=name, PolicyName=policy_name)


class GroupPolicy(Policy):
    @staticmethod
    def _iam_type():
        return "group"

    def _list(self, name):
        return self.client.list_group_policies(aws_retry=True, GroupName=name)

    def _get(self, name, policy_name):
        return self.client.get_group_policy(aws_retry=True, GroupName=name, PolicyName=policy_name)


def main():
    argument_spec = dict(
        iam_type=dict(required=True, choices=["user", "group", "role"]),
        iam_name=dict(required=True),
        policy_name=dict(default=None, required=False),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    args = dict(
        client=module.client("iam", retry_decorator=AWSRetry.jittered_backoff()),
        name=module.params.get("iam_name"),
        policy_name=module.params.get("policy_name"),
    )
    iam_type = module.params.get("iam_type")

    try:
        if iam_type == "user":
            policy = UserPolicy(**args)
        elif iam_type == "role":
            policy = RolePolicy(**args)
        elif iam_type == "group":
            policy = GroupPolicy(**args)

        module.exit_json(**(policy.run()))
    except is_boto3_error_code("NoSuchEntity") as e:
        module.exit_json(changed=False, msg=e.response["Error"]["Message"])
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)


if __name__ == "__main__":
    main()



# Copyright (c) 2025, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.general.plugins.module_utils import _deps as deps

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


with deps.declare("packaging"):
    from packaging.requirements import Requirement
    from packaging.version import InvalidVersion
    from packaging.version import parse as parse_version


class PackageRequirement:
    def __init__(self, module: AnsibleModule, name: str) -> None:
        self.module = module
        self.parsed_name, self.requirement = self._parse_spec(name)

    def _parse_spec(self, name: str) -> tuple[str, Requirement | None]:
        """
        Parse a package name that may include version specifiers using PEP 508.
        Returns a tuple of (name, requirement) where requirement is of type packaging.requirements.Requirement and it may be None.

        Example inputs:
            "package"
            "package>=1.0"
            "package>=1.0,<2.0"
            "package[extra]>=1.0"
            "package[foo,bar]>=1.0,!=1.5"

        :param name: Package name with optional version specifiers and extras
        :return: Tuple of (name, requirement)
        :raises ValueError: If the package specification is invalid
        """
        if not name:
            return name, None

        # Quick check for simple package names
        if not any(c in name for c in ">=<!~[]"):
            return name.strip(), None

        deps.validate(self.module, "packaging")

        try:
            req = Requirement(name)
            return req.name, req

        except Exception as e:
            raise ValueError(f"Invalid package specification for '{name}': {e}") from e

    def matches_version(self, version: str):
        """
        Check if a version string fulfills a version specifier.

        :param version: Version string to check
        :return: True if version fulfills the requirement, False otherwise
        :raises ValueError: If version is invalid
        """
        # If no spec provided, any version is valid
        if not self.requirement:
            return True

        try:
            # Parse version string
            ver = parse_version(version)

            return ver in self.requirement.specifier

        except InvalidVersion as e:
            raise ValueError(f"Invalid version '{version}': {e}") from e



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The vlan_parser plugin code
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible.errors import AnsibleFilterError


def _raise_error(msg):
    raise AnsibleFilterError(msg)


def vlan_parser(data, first_line_len=48, other_line_len=44):
    """
    Input: Unsorted list of vlan integers
    Output: Sorted string list of integers according to IOS-like vlan list rules

    1. Vlans are listed in ascending order
    2. Runs of 3 or more consecutive vlans are listed with a dash
    3. The first line of the list can be first_line_len characters long
    4. Subsequent list lines can be other_line_len characters
    """
    if not isinstance(data, (list)):
        _raise_error("Input is not valid for vlan_parser")
    # Sort and remove duplicates
    sorted_list = sorted(set(data))

    if not sorted_list:
        return []

    if sorted_list[0] < 1 or sorted_list[-1] > 4094:
        _raise_error("Valid VLAN range is 1-4094")

    parse_list = []
    idx = 0
    while idx < len(sorted_list):
        start = idx
        end = start
        while end < len(sorted_list) - 1:
            if sorted_list[end + 1] - sorted_list[end] == 1:
                end += 1
            else:
                break

        if start == end:
            # Single VLAN
            parse_list.append(str(sorted_list[idx]))
        elif start + 1 == end:
            # Run of 2 VLANs
            parse_list.append(str(sorted_list[start]))
            parse_list.append(str(sorted_list[end]))
        else:
            # Run of 3 or more VLANs
            parse_list.append(str(sorted_list[start]) + "-" + str(sorted_list[end]))
        idx = end + 1

    line_count = 0
    result = [""]
    for vlans in parse_list:
        # First line (" switchport trunk allowed vlan ")
        if line_count == 0:
            if len(result[line_count] + vlans) > first_line_len:
                result.append("")
                line_count += 1
                result[line_count] += vlans + ","
            else:
                result[line_count] += vlans + ","

        # Subsequent lines (" switchport trunk allowed vlan add ")
        else:
            if len(result[line_count] + vlans) > other_line_len:
                result.append("")
                line_count += 1
                result[line_count] += vlans + ","
            else:
                result[line_count] += vlans + ","

    # Remove trailing orphan commas
    for idx in range(0, len(result)):
        result[idx] = result[idx].rstrip(",")

    # Sometimes text wraps to next line, but there are no remaining VLANs
    if "" in result:
        result.remove("")

    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_clients_bandwidth_usage_history_info
short_description: Information module for organizations _clients _bandwidth _usage
  _history
description:
  - Get all organizations _clients _bandwidth _usage _history. - > Return data usage
    in megabits per second over time for all clients in the given organization within
    a given time range.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  networkTag:
    description:
      - NetworkTag query parameter. Match result to an exact network tag.
    type: str
  deviceTag:
    description:
      - DeviceTag query parameter. Match result to an exact device tag.
    type: str
  ssidName:
    description:
      - SsidName query parameter. Filter results by ssid name.
    type: str
  usageUplink:
    description:
      - UsageUplink query parameter. Filter results by usage uplink.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 186 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 186 days. The default is 1 day.
    type: float
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationClientsBandwidthUsageHistory
    description: Complete reference of the getOrganizationClientsBandwidthUsageHistory
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-clients-bandwidth-usage-history
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_clients_bandwidth_usage_history,
  - Paths used are
    get /organizations/{organizationId}/clients/bandwidthUsageHistory,
"""

EXAMPLES = r"""
- name: Get all organizations _clients _bandwidth _usage _history
  cisco.meraki.organizations_clients_bandwidth_usage_history_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkTag: string
    deviceTag: string
    ssidName: string
    usageUplink: string
    t0: string
    t1: string
    timespan: 0
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "ts": "string",
        "total": 0,
        "upstream": 0,
        "downstream": 0
      }
    ]
"""



#!/usr/bin/python

# Copyright (c) 2020 Jose Angel Munoz (@imjoseangel)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_stack_task_info
author: "Jose Angel Munoz (@imjoseangel)"
short_description: Return information of the tasks on a docker stack
description:
  - Retrieve information on docker stacks tasks using the C(docker stack) command on the target node (see examples).
extends_documentation_fragment:
  - community.docker._docker.cli_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
  - community.docker._attributes.info_module
  - community.docker._attributes.idempotent_not_modify_state
attributes:
  action_group:
    version_added: 3.6.0
options:
  name:
    description:
      - Stack name.
    type: str
    required: true
  docker_cli:
    version_added: 3.6.0
  docker_host:
    version_added: 3.6.0
  tls_hostname:
    version_added: 3.6.0
  api_version:
    version_added: 3.6.0
  ca_path:
    version_added: 3.6.0
  client_cert:
    version_added: 3.6.0
  client_key:
    version_added: 3.6.0
  tls:
    version_added: 3.6.0
  validate_certs:
    version_added: 3.6.0
  cli_context:
    version_added: 3.6.0
requirements:
  - Docker CLI tool C(docker)
"""

RETURN = r"""
results:
  description:
    - List of dictionaries containing the list of tasks associated to a stack name.
  sample:
    - CurrentState: Running
      DesiredState: Running
      Error: ""
      ID: 7wqv6m02ugkw
      Image: busybox
      Name: test_stack.1
      Node: swarm
      Ports: ""
  returned: always
  type: list
  elements: dict
"""

EXAMPLES = r"""
---
- name: Shows stack info
  community.docker.docker_stack_task_info:
    name: test_stack
  register: result

- name: Show results
  ansible.builtin.debug:
    var: result.results
"""

import json
import traceback

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.docker.plugins.module_utils._common_cli import (
    AnsibleModuleDockerClient,
    DockerException,
)


def main() -> None:
    client = AnsibleModuleDockerClient(
        argument_spec={"name": {"type": "str", "required": True}},
        supports_check_mode=True,
    )

    try:
        name = client.module.params["name"]
        rc, ret, stderr = client.call_cli_json_stream(
            "stack", "ps", name, "--format={{json .}}", check_rc=True
        )
        client.module.exit_json(
            changed=False,
            rc=rc,
            stdout="\n".join([json.dumps(entry) for entry in ret]),
            stderr=to_text(stderr).strip(),
            results=ret,
        )
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.afdorigingroupsoperations?view=azure-python
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_afdorigingroup_info
version_added: "3.4.0"
short_description: Get Azure Front Door OriginGroup facts to be used with Standard or Premium Frontdoor Service
description:
    - Get facts for a specific Azure Front Door (AFD) OriginGroup or all AFD OriginGroups.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    name:
        description:
            - Limit results to a specific AFD OriginGroup.
        type: str
    profile_name:
        description:
            - Name of the AFD profile.
        required: true
        type: str
    resource_group:
        description:
            - Name of the resource group where this AFD Profile belongs.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Get facts for all OriginGroups in AFD Profile
  azure_rm_afdorigingroup_info:
    resource_group: myResourceGroup
    profile_name: myCDNProfile

- name: Get facts of specific AFD OriginGroup
  azure_rm_afdorigingroup_info:
    name: myOriginGroup1
    profile_name: myCDNProfile
    resource_group: myResourceGroup
'''

RETURN = '''
afdorigingroups:
    description: List of AFD OriginGroups.
    returned: always
    type: complex
    contains:
        deployment_status:
            description:
                - Current state of the resource.
            type: str
            sample: NotStarted
        health_probe_settings:
            description:
                - Health probe settings to the origin that is used to determine the health of the origin.
            type: dict
            contains:
                probe_interval_in_seconds:
                    description:
                        - The number of seconds between health probes.
                    type: int
                probe_path:
                    description:
                        - The path relative to the origin that is used to determine the health of the origin.
                    type: str
                probe_protocol:
                    description:
                        - Protocol to use for health probe.
                    type: str
                    choices:
                        - Http
                        - Https
                        - NotSet
                probe_request_type:
                    description:
                        - The type of health probe request that is made.
                    type: str
                    choices:
                        - GET
                        - HEAD
                        - NotSet
        id:
            description:
                - ID of the AFD OriginGroup.
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myCDN/providers/Microsoft.Cdn/profiles/myProfile/origingroups/myOG1"
        load_balancing_settings:
            description:
                - Load balancing settings for a backend pool.
            type: dict
            contains:
                additional_latency_in_milliseconds:
                    description:
                        - The additional latency in milliseconds for probes to fall into the lowest latency bucket.
                    type: int
                sample_size:
                    description:
                        - The number of samples to consider for load balancing decisions.
                    type: int
                successful_samples_required:
                    description:
                        - The number of samples within the sample period that must succeed.
                    type: int
        name:
            description:
                - Name of the AFD OriginGroup.
            type: str
        profile_name:
            description:
                - Name of the AFD Profile where the OriginGroup will be added.
            type: str
        provisioning_state:
            description:
                - Provisioning status of the AFD OriginGroup.
            type: str
            sample: Succeeded
        resource_group:
            description:
                - Name of a resource group where the AFD OriginGroup exists or will be created.
            type: str
        session_affinity_state:
            description:
                - Whether to allow session affinity on this host.
            type: str
            choices:
                - Enabled
                - Disabled
        type:
            description:
                - Resource type.
            type: str
            sample: Microsoft.Cdn/profiles/afdorigingroups
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
import re

AZURE_OBJECT_CLASS = 'AFDOriginGroup'


class AzureRMAFDOriginGroupInfo(AzureRMModuleBase):
    """Utility class to get Azure AFD OriginGroup facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(
                type='str'
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            )
        )

        self.results = dict(
            changed=False,
            afdorigingroups=[]
        )

        self.name = None
        self.resource_group = None
        self.profile_name = None

        super(AzureRMAFDOriginGroupInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['afdorigingroups'] = self.get_item()
        else:
            self.results['afdorigingroups'] = self.list_by_profile()

        return self.results

    def get_item(self):
        """Get a single Azure AFD OriginGroup"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.cdn_client.afd_origin_groups.get(self.resource_group, self.profile_name, self.name)
        except Exception:
            pass

        if item:
            result = [self.serialize_afdorigingroup(item)]

        return result

    def list_by_profile(self):
        """Get all Azure AFD OriginGroups within an AFD profile"""

        self.log('List all AFD OriginGroups within an AFD profile')

        try:
            response = self.cdn_client.afd_origin_groups.list_by_profile(self.resource_group, self.profile_name)
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            results.append(self.serialize_afdorigingroup(item))

        return results

    def serialize_afdorigingroup(self, afdorigingroup):
        '''
        Convert a AFD OriginGroup object to dict.
        :param afdorigingroup: AFD OriginGroup object
        :return: dict
        '''
        result = self.serialize_obj(afdorigingroup, AZURE_OBJECT_CLASS)

        new_result = {}
        new_result['deployment_status'] = afdorigingroup.deployment_status
        new_result['health_probe_settings'] = dict()
        new_result['health_probe_settings']['probe_interval_in_seconds'] = afdorigingroup.health_probe_settings.probe_interval_in_seconds
        new_result['health_probe_settings']['probe_path'] = afdorigingroup.health_probe_settings.probe_path
        new_result['health_probe_settings']['probe_protocol'] = afdorigingroup.health_probe_settings.probe_protocol
        new_result['health_probe_settings']['probe_request_type'] = afdorigingroup.health_probe_settings.probe_request_type
        new_result['id'] = afdorigingroup.id
        new_result['load_balancing_settings'] = dict()
        new_result['load_balancing_settings']['additional_latency_in_milliseconds'] = afdorigingroup.load_balancing_settings.additional_latency_in_milliseconds
        new_result['load_balancing_settings']['sample_size'] = afdorigingroup.load_balancing_settings.sample_size
        new_result['load_balancing_settings']['successful_samples_required'] = afdorigingroup.load_balancing_settings.successful_samples_required
        new_result['name'] = afdorigingroup.name
        new_result['profile_name'] = re.sub('\\/.*', '', re.sub('.*profiles\\/', '', result['id']))
        new_result['provisioning_state'] = afdorigingroup.provisioning_state
        new_result['resource_group'] = re.sub('\\/.*', '', re.sub('.*resourcegroups\\/', '', result['id']))
        new_result['session_affinity_state'] = afdorigingroup.session_affinity_state
        new_result['type'] = afdorigingroup.type
        return new_result


def main():
    """Main module execution code path"""
    AzureRMAFDOriginGroupInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: subnet_of
"""

from __future__ import absolute_import, division, print_function

from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _is_subnet_of,
    _need_ipaddress,
    _validate_args,
    ip_network,
)


__metaclass__ = type

DOCUMENTATION = """
    name: subnet_of
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if a network is a subnet of another network
    description:
        - This plugin checks if the first network is a subnet of the second network amongst the provided network addresses
    options:
        network_a:
            description:
            - A string that represents the first network address
            - 'For example: C(10.1.1.0/24)'
            type: str
            required: True
        network_b:
            description:
            - A string that represents the second network address
            - 'For example: C(10.0.0.0/8)'
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

- name: Check if 10.1.1.0/24 is a subnet of 10.0.0.0/8
  ansible.builtin.set_fact:
    data: "{{ '10.1.1.0/24' is ansible.utils.subnet_of '10.0.0.0/8' }}"

# TASK [Check if 10.1.1.0/24 is a subnet of 10.0.0.0/8] **************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 192.168.1.0/24 is not a subnet of 10.0.0.0/8
  ansible.builtin.set_fact:
    data: "{{ '192.168.1.0/24' is not ansible.utils.subnet_of '10.0.0.0/8' }}"

# TASK [Check if 192.168.1.0/24 is not a subnet of 10.0.0.0/8] *******************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }
"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


@_need_ipaddress
def _subnet_of(network_a, network_b):
    """Test if a network is a subnet of another network"""

    params = {"network_a": network_a, "network_b": network_b}
    _validate_args("subnet_of", DOCUMENTATION, params)

    try:
        return _is_subnet_of(ip_network(network_a), ip_network(network_b))
    except Exception:
        return False


class TestModule(object):
    """network jinja test"""

    test_map = {"subnet_of": _subnet_of}

    def tests(self):
        return self.test_map



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_rpm
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Install patch or feature rpms on Cisco NX-OS devices.
description:
- Install software maintenance upgrade (smu) RPMS and 3rd party RPMS on Cisco NX-OS
  devices.
version_added: 1.0.0
author: Sai Chintalapudi (@saichint)
notes:
- Tested against NXOSv 7.0(3)I2(5), 7.0(3)I4(6), 7.0(3)I5(3), 7.0(3)I6(1), 7.0(3)I7(3)
- Unsupported for Cisco MDS
- For patches, the minimum platform version needed is 7.0(3)I2(5)
- For feature rpms, the minimum platform version needed is 7.0(3)I6(1)
- The module manages the entire RPM lifecycle (Add, activate, commit, deactivate,
  remove)
- For reload patches, this module is NOT idempotent until the patch is committed.
options:
  pkg:
    description:
    - Name of the RPM package.
    type: str
  file_system:
    description:
    - The remote file system of the device. If omitted, devices that support a file_system
      parameter will use their default values.
    default: bootflash
    type: str
  aggregate:
    description:
    - List of RPM/patch definitions.
    type: list
    elements: dict
    suboptions:
      pkg:
        description:
        - Name of the RPM package.
        required: True
        type: str
      file_system:
        description:
        - The remote file system of the device. If omitted, devices that support a file_system
          parameter will use their default values.
        type: str
      state:
        description:
        - If the state is present, the rpm will be installed, If the state is absent,
          it will be removed.
        choices:
        - present
        - absent
        type: str
  state:
    description:
    - If the state is present, the rpm will be installed, If the state is absent,
      it will be removed.
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
- cisco.nxos.nxos_rpm:
    pkg: nxos.sample-n9k_ALL-1.0.0-7.0.3.I7.3.lib32_n9000.rpm
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["install add bootflash:nxos.sample-n9k_ALL-1.0.0-7.0.3.I7.3.lib32_n9000.rpm forced",
             "install activate nxos.sample-n9k_ALL-1.0.0-7.0.3.I7.3.lib32_n9000 forced",
             "install commit nxos.sample-n9k_ALL-1.0.0-7.0.3.I7.3.lib32_n9000"]
"""


import time

from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)


def execute_show_command(command, module):
    iteration = 0
    cmds = [{"command": command, "output": "text"}]

    while iteration < 10:
        body = run_commands(module, cmds)[0]
        if body:
            return body
        else:
            time.sleep(2)
            iteration += 1


def remote_file_exists(module, dst, file_system):
    command = "dir {0}:/{1}".format(file_system, dst)
    body = execute_show_command(command, module)
    if "No such file" in body:
        return False
    return True


def config_cmd_operation(module, cmd):
    iteration = 0
    while iteration < 10:
        msg = load_config(module, [cmd], True)
        if msg:
            if (
                "another install operation is in progress" in msg[0].lower()
                or "failed" in msg[0].lower()
            ):
                time.sleep(2)
                iteration += 1
            else:
                return
        else:
            return


def validate_operation(module, show_cmd, cfg_cmd, pkg, pkg_not_present):
    iteration = 0
    while iteration < 10:
        body = execute_show_command(show_cmd, module)
        if pkg_not_present:
            if pkg not in body:
                return
        else:
            if pkg in body:
                return
        time.sleep(2)
        iteration += 1

    err = 'Operation "{0}" Failed'.format(cfg_cmd)
    module.fail_json(msg=err)


def add_operation(module, show_cmd, file_system, full_pkg, pkg):
    cmd = "install add {0}:{1}".format(file_system, full_pkg)
    config_cmd_operation(module, cmd)
    validate_operation(module, show_cmd, cmd, pkg, False)
    return cmd


def activate_operation(module, show_cmd, pkg):
    cmd = "install activate {0} forced".format(pkg)
    config_cmd_operation(module, cmd)
    validate_operation(module, show_cmd, cmd, pkg, False)
    return cmd


def activate_reload(module, pkg, flag):
    iteration = 0
    if flag:
        cmd = "install activate {0} forced".format(pkg)
    else:
        cmd = "install deactivate {0} forced".format(pkg)
    opts = {"ignore_timeout": True}
    while iteration < 10:
        msg = load_config(module, [cmd], True, opts)
        if msg:
            if isinstance(msg[0], int):
                if msg[0] == -32603:
                    return cmd
            elif isinstance(msg[0], str):
                if "socket is closed" in msg[0].lower():
                    return cmd
                if (
                    "another install operation is in progress" in msg[0].lower()
                    or "failed" in msg[0].lower()
                ):
                    time.sleep(2)
        iteration += 1


def commit_operation(module, show_cmd, pkg, flag):
    cmd = "install commit {0}".format(pkg)
    config_cmd_operation(module, cmd)
    validate_operation(module, show_cmd, cmd, pkg, flag)
    return cmd


def deactivate_operation(module, show_cmd, pkg, flag):
    cmd = "install deactivate {0} forced".format(pkg)
    config_cmd_operation(module, cmd)
    validate_operation(module, show_cmd, cmd, pkg, flag)
    return cmd


def terminal_operation(module, config):
    if config:
        cmd = "terminal dont-ask"
    else:
        cmd = "no terminal dont-ask"
    config_cmd_operation(module, cmd)
    return cmd


def remove_operation(module, show_cmd, pkg):
    commands = []
    commands.append(terminal_operation(module, True))
    cmd = "install remove {0} forced".format(pkg)
    config_cmd_operation(module, cmd)
    validate_operation(module, show_cmd, cmd, pkg, True)
    commands.append(cmd)
    commands.append(terminal_operation(module, False))
    return commands


def install_remove_rpm(module, full_pkg, file_system, state):
    commands = []
    reload_patch = False

    splitted_pkg = full_pkg.split(".")
    pkg = ".".join(splitted_pkg[0:-1])

    show_inactive = "show install inactive"
    show_active = "show install active"
    show_commit = "show install committed"
    show_patches = "show install patches"
    show_pkg_info = "show install pkg-info {0}".format(pkg)

    if state == "present":
        inactive_body = execute_show_command(show_inactive, module)
        active_body = execute_show_command(show_active, module)

        if pkg not in inactive_body and pkg not in active_body:
            commands.append(add_operation(module, show_inactive, file_system, full_pkg, pkg))

        patch_type_body = execute_show_command(show_pkg_info, module)
        if patch_type_body and "Patch Type    :  reload" in patch_type_body:
            # This is reload smu/patch rpm
            reload_patch = True

        if pkg not in active_body:
            if reload_patch:
                commands.append(activate_reload(module, pkg, True))
                return commands
            else:
                commands.append(activate_operation(module, show_active, pkg))

        commit_body = execute_show_command(show_commit, module)
        if pkg not in commit_body:
            patch_body = execute_show_command(show_patches, module)
            if pkg in patch_body:
                # This is smu/patch rpm
                commands.append(commit_operation(module, show_commit, pkg, False))
            else:
                err = 'Operation "install activate {0} forced" Failed'.format(pkg)
                module.fail_json(msg=err)

    else:
        commit_body = execute_show_command(show_commit, module)
        active_body = execute_show_command(show_active, module)

        patch_type_body = execute_show_command(show_pkg_info, module)
        if patch_type_body and "Patch Type    :  reload" in patch_type_body:
            # This is reload smu/patch rpm
            reload_patch = True

        if pkg in commit_body and pkg in active_body:
            if reload_patch:
                commands.append(activate_reload(module, pkg, False))
                return commands
            else:
                commands.append(deactivate_operation(module, show_active, pkg, True))
                commit_body = execute_show_command(show_commit, module)
                if pkg in commit_body:
                    # This is smu/patch rpm
                    commands.append(commit_operation(module, show_commit, pkg, True))
                commands.extend(remove_operation(module, show_inactive, pkg))

        elif pkg in commit_body:
            # This is smu/patch rpm
            commands.append(commit_operation(module, show_commit, pkg, True))
            commands.extend(remove_operation(module, show_inactive, pkg))

        elif pkg in active_body:
            # This is smu/patch rpm
            if reload_patch:
                commands.append(activate_reload(module, pkg, False))
                return commands
            else:
                commands.append(deactivate_operation(module, show_inactive, pkg, False))
                commands.extend(remove_operation(module, show_inactive, pkg))

        else:
            inactive_body = execute_show_command(show_inactive, module)
            if pkg in inactive_body:
                commands.extend(remove_operation(module, show_inactive, pkg))

    return commands


def main():
    element_spec = dict(
        pkg=dict(type="str"),
        file_system=dict(type="str", default="bootflash"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    aggregate_spec = deepcopy(element_spec)
    aggregate_spec["pkg"] = dict(required=True)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    argument_spec = dict(aggregate=dict(type="list", elements="dict", options=aggregate_spec))

    argument_spec.update(element_spec)
    required_one_of = [["pkg", "aggregate"]]
    mutually_exclusive = [["pkg", "aggregate"]]

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=required_one_of,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=False,
    )

    warnings = list()
    results = {"changed": False, "commands": [], "warnings": warnings}

    aggregate = module.params.get("aggregate")
    objects = []
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]

            d = item.copy()
            objects.append(d)
    else:
        objects.append(
            {
                "pkg": module.params["pkg"],
                "file_system": module.params["file_system"],
                "state": module.params["state"],
            },
        )

    for obj in objects:
        if obj["state"] == "present":
            remote_exists = remote_file_exists(module, obj["pkg"], file_system=obj["file_system"])

            if not remote_exists:
                module.fail_json(msg="The requested package doesn't exist on the device")

        cmds = install_remove_rpm(module, obj["pkg"], obj["file_system"], obj["state"])

        if cmds:
            results["changed"] = True
            results["commands"].extend(cmds)

    module.exit_json(**results)


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import socket
import typing as t

from .._import_helper import urllib3
from ..errors import DockerException

if t.TYPE_CHECKING:
    from requests import Response

_T = t.TypeVar("_T")


class CancellableStream(t.Generic[_T]):
    """
    Stream wrapper for real-time events, logs, etc. from the server.

    Example:
        >>> events = client.events()
        >>> for event in events:
        ...   print(event)
        >>> # and cancel from another thread
        >>> events.close()
    """

    def __init__(self, stream: t.Generator[_T], response: Response) -> None:
        self._stream = stream
        self._response = response

    def __iter__(self) -> t.Self:
        return self

    def __next__(self) -> _T:
        try:
            return next(self._stream)
        except urllib3.exceptions.ProtocolError as exc:
            raise StopIteration from exc
        except socket.error as exc:
            raise StopIteration from exc

    next = __next__

    def close(self) -> None:
        """
        Closes the event streaming.
        """

        if not self._response.raw.closed:
            # find the underlying socket object
            # based on api.client._get_raw_response_socket

            sock_fp = self._response.raw._fp.fp  # type: ignore

            if hasattr(sock_fp, "raw"):
                sock_raw = sock_fp.raw

                if hasattr(sock_raw, "sock"):
                    sock = sock_raw.sock

                elif hasattr(sock_raw, "_sock"):
                    sock = sock_raw._sock

            elif hasattr(sock_fp, "channel"):
                # We are working with a paramiko (SSH) channel, which does not
                # support cancelable streams with the current implementation
                raise DockerException(
                    "Cancellable streams not supported for the SSH protocol"
                )
            else:
                sock = sock_fp._sock  # type: ignore

            if hasattr(urllib3.contrib, "pyopenssl") and isinstance(
                sock, urllib3.contrib.pyopenssl.WrappedSocket
            ):
                sock = sock.socket

            sock.shutdown(socket.SHUT_RDWR)
            sock.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_data_type_traditional_group_facts
short_description: Get data-type-traditional-group objects facts on Checkpoint over Web Services API
description:
  - Get data-type-traditional-group objects facts on Checkpoint devices.
  - All operations are performed over Web Services API.
  - This module handles both operations, get a specific object and get several objects,
    For getting a specific object use the parameter 'name'.
  - Available from R82 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
        This parameter is relevant only for getting a specific object.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  filter:
    description:
      - Search expression to filter objects by. The provided text should be exactly the same as it would be given in SmartConsole Object Explorer. The
        logical operators in the expression ('AND', 'OR') should be provided in capital letters. The search involves both a IP search and a textual search in
        name, comment, tags etc.
    type: str
  limit:
    description:
      - The maximal number of returned results.
        This parameter is relevant only for getting few objects.
    type: int
  offset:
    description:
      - Number of the results to initially skip.
        This parameter is relevant only for getting few objects.
    type: int
  order:
    description:
      - Sorts the results by search criteria. Automatically sorts the results by Name, in the ascending order.
        This parameter is relevant only for getting few objects.
    type: list
    elements: dict
    suboptions:
      ASC:
        description:
          - Sorts results by the given field in ascending order.
        type: str
        choices: ['name']
      DESC:
        description:
          - Sorts results by the given field in descending order.
        type: str
        choices: ['name']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
extends_documentation_fragment: check_point.mgmt.checkpoint_facts
"""

EXAMPLES = """
- name: show-data-type-traditional-group
  cp_mgmt_data_type_traditional_group_facts:
    name: trad-group-obj

- name: show-data-type-traditional-groups
  cp_mgmt_data_type_traditional_group_facts:
"""

RETURN = """
ansible_facts:
  description: The checkpoint object facts.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_facts, api_call_facts


def main():
    argument_spec = dict(
        name=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        filter=dict(type='str'),
        limit=dict(type='int'),
        offset=dict(type='int'),
        order=dict(type='list', elements='dict', options=dict(
            ASC=dict(type='str', choices=['name']),
            DESC=dict(type='str', choices=['name'])
        )),
        domains_to_process=dict(type='list', elements='str')
    )
    argument_spec.update(checkpoint_argument_spec_for_facts)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    api_call_object = "data-type-traditional-group"
    api_call_object_plural_version = "data-type-traditional-groups"

    result = api_call_facts(module, api_call_object, api_call_object_plural_version)
    module.exit_json(ansible_facts=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Gu Fred-Sun, (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_vpnsite
version_added: '1.5.0'
short_description: Manage Azure VpnSite instance
description:
    - Create, update and delete instance of Azure VpnSite.
options:
    resource_group:
        description:
            - The resource group name of the VpnSite.
        required: true
        type: str
    location:
        description:
            - The location of the VpnSite
        type: str
    name:
        description:
            - The name of the VpnSite.
        required: true
        type: str
    virtual_wan:
        description:
            - The VirtualWAN to which the vpnSite belongs.
        type: dict
        suboptions:
            id:
                description:
                    - The resource ID of the related virtual wan.
                type: str
    device_properties:
        description:
            - The device properties.
        type: dict
        suboptions:
            device_vendor:
                description:
                    - Name of the device Vendor.
                type: str
            device_model:
                description:
                    - Model of the device.
                type: str
            link_speed_in_mbps:
                description:
                    - Link speed.
                type: int
    ip_address:
        description:
            - The ip-address for the vpn-site.
        type: str
    site_key:
        description:
            - The key for vpn-site that can be used for connections.
        type: str
    address_space:
        description:
            - The AddressSpace that contains an array of IP address ranges.
        type: dict
        suboptions:
            address_prefixes:
                description:
                    - A list of address blocks reserved for this virtual network in CIDR notation.
                type: list
                elements: str
    bgp_properties:
        description:
            - The set of bgp properties.
        type: dict
        suboptions:
            asn:
                description:
                    - The BGP speaker's ASN.
                type: int
            bgp_peering_address:
                description:
                    - The BGP peering address and BGP identifier of this BGP speaker.
                type: str
            peer_weight:
                description:
                    - The weight added to routes learned from this BGP speaker.
                type: int
            bgp_peering_addresses:
                description:
                    - BGP peering address with IP configuration ID for virtual network gateway.
                type: list
                elements: dict
                suboptions:
                    ipconfiguration_id:
                        description:
                            - The ID of IP configuration which belongs to gateway.
                        type: str
                    default_bgp_ip_addresses:
                        description:
                            - The list of default BGP peering addresses which belong to IP configuration.
                        type: list
                        elements: str
                    custom_bgp_ip_addresses:
                        description:
                            - The list of custom BGP peering addresses which belong to IP configuration.
                        type: list
                        elements: str
                    tunnel_ip_addresses:
                        description:
                            - The list of tunnel public IP addresses which belong to IP configuration.
                        type: list
                        elements: str
    is_security_site:
        description:
            - IsSecuritySite flag.
        type: bool
    vpn_site_links:
        description:
            - List of all vpn site links.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the resource that is unique within a resource group.
                    - This name can be used to access the resource.
                type: str
            link_properties:
                description:
                    - The link provider properties.
                type: dict
                suboptions:
                    link_provider_name:
                        description:
                            - Name of the link provider.
                        type: str
                    link_speed_in_mbps:
                        description:
                            - Link speed.
                        type: int
            ip_address:
                description:
                    - The IP address for the vpn site link.
                type: str
            fqdn:
                description:
                    - FQDN of vpn-site-link.
                type: str
            bgp_properties:
                description:
                    - The set of bgp properties.
                type: dict
                suboptions:
                    asn:
                        description:
                            - The BGP speaker's ASN.
                        type: int
                    bgp_peering_address:
                        description:
                            - The BGP peering address and BGP identifier of this BGP speaker.
                        type: str
    o365_policy:
        description:
            - Office365 Policy.
        type: dict
        suboptions:
            break_out_categories:
                description:
                    - Office365 breakout categories.
                type: dict
                suboptions:
                    allow:
                        description:
                            - Flag to control allow category.
                        type: bool
                    optimize:
                        description:
                            - Flag to control optimize category.
                        type: bool
                    default:
                        description:
                            - Flag to control default category.
                        type: bool
    state:
        description:
            - Assert the state of the VpnSite.
            - Use C(present) to create or update an VpnSite and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Fred-Sun (@Fred-Sun)

'''

EXAMPLES = '''
- name: Create VpnSite
  azure_rm_vpnsite:
    resource_group: myResourceGroup
    name: vpnSite_name

- name: Delete Vpn Site
  azure_rm_vpnsite:
    resource_group: myResourceGroup
    name: vpnSite_name
'''

RETURN = '''
state:
    description:
        - Current state of the vpn site.
    type: complex
    returned: success
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/vpnSites/vpn_site_name
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: vpn_site_name
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: Microsoft.Network/vpnSites
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'key1': 'value1'}
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: 8d7415fe-d92c-4331-92ea-460aadfb9648
        virtual_wan:
            description:
                - The VirtualWAN to which the vpnSite belongs.
            returned: always
            type: complex
            contains:
                id:
                    description:
                        - Resource ID.
                    returned: always
                    type: str
                    sample: /subscriptions/xxx-xxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/virtualWans/virtualwan_name
        device_properties:
            description:
                - The device properties.
            returned: always
            type: complex
            contains:
                device_vendor:
                    description:
                        - Name of the device Vendor.
                    returned: always
                    type: str
                    sample: {"link_speed_in_mbps": 0}
        provisioning_state:
            description:
                - The provisioning state of the VPN site resource.
            returned: always
            type: str
            sample: "Succeeded"
        is_security_site:
            description:
                - IsSecuritySite flag.
            returned: always
            type: bool
            sample: false
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete, Update_tags = range(5)


class AzureRMVpnSite(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            virtual_wan=dict(
                type='dict',
                options=dict(
                    id=dict(
                        type='str',
                    )
                )
            ),
            device_properties=dict(
                type='dict',
                options=dict(
                    device_vendor=dict(
                        type='str',
                    ),
                    device_model=dict(
                        type='str',
                    ),
                    link_speed_in_mbps=dict(
                        type='int',
                    )
                )
            ),
            ip_address=dict(
                type='str',
            ),
            site_key=dict(
                type='str',
                no_log=True,
            ),
            address_space=dict(
                type='dict',
                options=dict(
                    address_prefixes=dict(
                        type='list',
                        elements='str'
                    )
                )
            ),
            bgp_properties=dict(
                type='dict',
                options=dict(
                    asn=dict(
                        type='int',
                    ),
                    bgp_peering_address=dict(
                        type='str',
                    ),
                    peer_weight=dict(
                        type='int',
                    ),
                    bgp_peering_addresses=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            ipconfiguration_id=dict(
                                type='str',
                            ),
                            default_bgp_ip_addresses=dict(
                                type='list',
                                elements='str'
                            ),
                            custom_bgp_ip_addresses=dict(
                                type='list',
                                elements='str'
                            ),
                            tunnel_ip_addresses=dict(
                                type='list',
                                elements='str'
                            )
                        )
                    )
                )
            ),
            is_security_site=dict(
                type='bool',
            ),
            vpn_site_links=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(
                        type='str',
                    ),
                    link_properties=dict(
                        type='dict',
                        options=dict(
                            link_provider_name=dict(
                                type='str',
                            ),
                            link_speed_in_mbps=dict(
                                type='int',
                            )
                        )
                    ),
                    ip_address=dict(
                        type='str',
                    ),
                    fqdn=dict(
                        type='str',
                    ),
                    bgp_properties=dict(
                        type='dict',
                        options=dict(
                            asn=dict(
                                type='int',
                            ),
                            bgp_peering_address=dict(
                                type='str',
                            )
                        )
                    )
                )
            ),
            o365_policy=dict(
                type='dict',
                options=dict(
                    break_out_categories=dict(
                        type='dict',
                        options=dict(
                            allow=dict(
                                type='bool',
                            ),
                            optimize=dict(
                                type='bool',
                            ),
                            default=dict(
                                type='bool',
                            )
                        )
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.body = {}

        self.results = dict(changed=False)
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMVpnSite, self).__init__(derived_arg_spec=self.module_arg_spec,
                                             supports_check_mode=True,
                                             supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        resource_group = self.get_resource_group(self.resource_group)
        if self.location is None:
            # Set default location
            self.location = resource_group.location
        self.body['location'] = self.location

        old_response = None
        response = None

        old_response = self.get_resource()

        if not old_response:
            if self.state == 'present':
                self.to_do = Actions.Create
        else:
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                if self.body.get('virtual_wan') is not None and self.body['virtual_wan'] != old_response.get('virtual_wan'):
                    self.to_do = Actions.Update
                for key in self.body.keys():
                    if key == 'address_space':
                        if old_response.get('address_space') is None or\
                           len(self.body['address_space']['address_prefixes']) > len(old_response['address_space']['address_prefixes']) or\
                           not all(key in old_response['address_space']['address_prefixes'] for key in self.body['address_space']['address_prefixes']):
                            self.to_do = Actions.Update
                    elif key == 'device_properties':
                        if old_response.get('device_properties') is None or\
                           not all(self.body['device_properties'][key] == old_response['device_properties'].get(key)
                           for key in self.body['device_properties'].keys()):
                            self.to_do = Actions.Update
                    elif key == 'o365_policy':
                        if old_response.get('o365_policy') is None or\
                           not all(self.body['o365_policy']['break_out_categories'][key] == old_response['o365_policy']['break_out_categories'].get(key)
                           for key in self.body['o365_policy']['break_out_categories'].keys()):
                            self.to_do = Actions.Update
                    elif key == 'vpn_site_links':
                        if old_response.get('vpn_site_links') is None or\
                           not all(self.body['vpn_site_links'][key] == old_response['vpn_site_links'].get(key)
                           for key in self.body['vpn_site_links'].keys()):
                            self.to_do = Actions.Update
                    elif key == 'bgp_properties':
                        if old_response.get('bgp_properties') is None:
                            self.to_do = Actions.Update
                        else:
                            for item in self.body['bgp_properties'].keys():
                                if item != 'bgp_peering_addresses' and item != 'peer_weight':
                                    if self.body['bgp_properties'][item] != old_response['bgp_properties'].get(item):
                                        self.to_do = Actions.Update
                                else:
                                    if self.body['bgp_properties'].get('bgp_peering_addresses') is not None:
                                        bgp_address = old_response['bgp_properties']['bgp_peering_addresses']
                                        if old_response['bgp_properties'].get('bgp_peering_addresses') is None or\
                                           not all(self.body['bgp_properties']['bgp_peering_addresses'][value] == bgp_address.get(value)
                                           for value in ['ipconfiguration_id', 'custom_bgp_ip_addresses']):
                                            self.to_do = Actions.Update

                    elif self.body[key] != old_response.get(key):
                        self.to_do = Actions.Update

                update_tags, self.tags = self.update_tags(old_response.get('tags'))
                if update_tags:
                    self.to_do = Actions.Update_tags

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_update_resource()
        elif self.to_do == Actions.Update_tags:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.update_resource_tags(dict(tags=self.tags))
        elif self.to_do == Actions.Delete:
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            self.delete_resource()
        else:
            self.results['changed'] = False
            response = old_response

        if response is not None:
            self.results['state'] = response
        return self.results

    def update_resource_tags(self, tags_parameters):
        try:
            response = self.network_client.vpn_sites.update_tags(resource_group_name=self.resource_group,
                                                                 vpn_site_name=self.name,
                                                                 vpn_site_parameters=tags_parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to update the VpnSite instance tags.')
            self.fail('Error updating the VpnSite instance: {0}'.format(str(exc)))
        return response.as_dict()

    def create_update_resource(self):
        try:
            response = self.network_client.vpn_sites.begin_create_or_update(resource_group_name=self.resource_group,
                                                                            vpn_site_name=self.name,
                                                                            vpn_site_parameters=self.body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.log('Error attempting to create the VpnSite instance.')
            self.fail('Error creating the VpnSite instance: {0}'.format(str(exc)))
        return response.as_dict()

    def delete_resource(self):
        try:
            response = self.network_client.vpn_sites.begin_delete(resource_group_name=self.resource_group,
                                                                  vpn_site_name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the VpnSite instance.')
            self.fail('Error deleting the VpnSite instance: {0}'.format(str(e)))

        return True

    def get_resource(self):
        try:
            response = self.network_client.vpn_sites.get(resource_group_name=self.resource_group,
                                                         vpn_site_name=self.name)
        except ResourceNotFoundError as e:
            return False
        return response.as_dict()


def main():
    AzureRMVpnSite()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2013, Jeroen Hoekx <jeroen.hoekx@dsquare.be>
# Copyright (c) 2016, Matt Robinson <git@nerdoftheherd.com>
# Copyright (c) 2017, Dag Wieers <dag@wieers.com>
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Jeroen Hoekx (@jhoekx)
  - Matt Robinson (@ribbons)
  - Dag Wieers (@dagwieers)
module: iso_extract
short_description: Extract files from an ISO image
description:
  - This module has two possible ways of operation.
  - If 7zip is installed on the system, this module extracts files from an ISO into a temporary directory and copies files
    to a given destination, if needed.
  - If the user has mount-capabilities (CAP_SYS_ADMIN on Linux) this module mounts the ISO image to a temporary location,
    and copies files to a given destination, if needed.
requirements:
  - Either 7z (from C(7zip) or C(p7zip) package)
  - Or mount capabilities (root-access, or CAP_SYS_ADMIN capability on Linux)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  image:
    description:
      - The ISO image to extract files from.
    type: path
    required: true
    aliases: [path, src]
  dest:
    description:
      - The destination directory to extract files to.
    type: path
    required: true
  files:
    description:
      - A list of files to extract from the image.
      - Extracting directories does not work.
      - File paths should not include a leading V(/); any leading path separator is automatically stripped.
    type: list
    elements: str
    required: true
  force:
    description:
      - If V(true), it replaces the remote file when contents are different than the source.
      - If V(false), the file is only extracted and copied if the destination does not already exist.
    type: bool
    default: true
  executable:
    description:
      - The path to the C(7z) executable to use for extracting files from the ISO.
      - If not provided, it assumes the value V(7z).
    type: path
  password:
    description:
      - Password used to decrypt files from the ISO.
      - It is only used if C(7z) is used.
      - The password is used as a command line argument to 7z. This is a B(potential security risk) that allows passwords
        to be revealed if someone else can list running processes on the same machine in the right moment.
    type: str
    version_added: 10.1.0
notes:
  - Only the file checksum (content) is taken into account when extracting files from the ISO image. If O(force=false), only
    checks the presence of the file.
"""

EXAMPLES = r"""
- name: Extract kernel and ramdisk from a LiveCD
  community.general.iso_extract:
    image: /tmp/rear-test.iso
    dest: /tmp/virt-rear/
    files:
      - isolinux/kernel
      - isolinux/initrd.cgz
"""

RETURN = r"""
#
"""

import os.path
import shutil
import tempfile
import time

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict(
            image=dict(type="path", required=True, aliases=["path", "src"]),
            dest=dict(type="path", required=True),
            files=dict(type="list", elements="str", required=True),
            force=dict(type="bool", default=True),
            password=dict(type="str", no_log=True),
            executable=dict(type="path"),  # No default on purpose
        ),
        supports_check_mode=True,
    )
    image = module.params["image"]
    dest = module.params["dest"]
    files = module.params["files"]
    force = module.params["force"]
    password = module.params["password"]
    executable = module.params["executable"]

    result = dict(
        changed=False,
        dest=dest,
        image=image,
    )

    # We want to know if the user provided it or not, so we set default here
    if executable is None:
        executable = "7z"

    binary = module.get_bin_path(executable, None)

    # When executable was provided and binary not found, warn user !
    if module.params["executable"] is not None and not binary:
        module.warn(f"Executable '{executable}' is not found on the system, trying to mount ISO instead.")

    if not os.path.exists(dest):
        module.fail_json(msg=f"Directory '{dest}' does not exist")

    if not os.path.exists(os.path.dirname(image)):
        module.fail_json(msg=f"ISO image '{image}' does not exist")

    result["files"] = []
    extract_files = [f.lstrip(os.sep) for f in files]

    if not force:
        # Check if we have to process any files based on existence
        for f in extract_files[:]:
            dest_file = os.path.join(dest, os.path.basename(f))
            if os.path.exists(dest_file):
                result["files"].append(
                    dict(
                        checksum=None,
                        dest=dest_file,
                        src=f,
                    )
                )
                extract_files.remove(f)

    if not extract_files:
        module.exit_json(**result)

    tmp_dir = tempfile.mkdtemp()

    # Use 7zip when we have a binary, otherwise try to mount
    if binary:
        cmd = [binary, "x", image, f"-o{tmp_dir}"]
        if password:
            cmd += [f"-p{password}"]
        cmd += extract_files
    else:
        cmd = [module.get_bin_path("mount"), "-o", "loop,ro", image, tmp_dir]

    rc, out, err = module.run_command(cmd)
    if rc != 0:
        result.update(
            dict(
                cmd=cmd,
                rc=rc,
                stderr=err,
                stdout=out,
            )
        )
        shutil.rmtree(tmp_dir)

        if binary:
            module.fail_json(msg=f"Failed to extract from ISO image '{image}' to '{tmp_dir}'", **result)
        else:
            module.fail_json(
                msg=f"Failed to mount ISO image '{image}' to '{tmp_dir}', and we could not find executable '{executable}'.",
                **result,
            )

    try:
        for f in extract_files:
            tmp_src = os.path.join(tmp_dir, f)
            if not os.path.exists(tmp_src):
                module.fail_json(msg=f"Failed to extract '{f}' from ISO image", **result)

            src_checksum = module.sha1(tmp_src)

            dest_file = os.path.join(dest, os.path.basename(f))

            if os.path.exists(dest_file):
                dest_checksum = module.sha1(dest_file)
            else:
                dest_checksum = None

            result["files"].append(
                dict(
                    checksum=src_checksum,
                    dest=dest_file,
                    src=f,
                )
            )

            if src_checksum != dest_checksum:
                if not module.check_mode:
                    shutil.copy(tmp_src, dest_file)

                result["changed"] = True
    finally:
        if not binary:
            umount_cmd = [module.get_bin_path("umount"), tmp_dir]
            for dummy in range(5):
                rc, dummy, dummy = module.run_command(umount_cmd)
                if rc == 0:
                    break
                time.sleep(1)
            else:
                module.warn(f"Failed to unmount ISO image from '{tmp_dir}' after 5 attempts.")

        shutil.rmtree(tmp_dir)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The L3_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class L3_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(L3_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""^interface
                    (\s(?P<name>\S+))
                    $""", re.VERBOSE,
            ),
            "compval": "name",
            "setval": "interface {{ name }}",
            "result": {"{{ name }}": {"name": "{{ name }}"}},
            "shared": True,
        },
        {
            "name": "mac_address",
            "getval": re.compile(
                r"""\s+mac-address
                    (\s(?P<mac_address>\S+))
                    $""", re.VERBOSE,
            ),
            "setval": "mac-address {{ mac_address }}",
            "result": {"{{ name }}": {"mac_address": "{{ mac_address }}"}},
        },
        {
            "name": "bandwidth",
            "getval": re.compile(
                r"""
                \s+bandwidth
                (\s+(?P<inherit>inherit))?
                (\s+(?P<kilobits>\d+))?
                \s*$""", re.VERBOSE,
            ),
            "setval": "bandwidth"
                      "{{ ' inherit' if bandwidth.inherit|d(False) else ''}}"
                      "{{ ' ' + bandwidth.kilobits|string if bandwidth.kilobits is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "bandwidth": {
                        "inherit": "{{ not not inherit }}",
                        "kilobits": "{{ kilobits }}",
                    },
                },
            },
        },
        {
            "name": "dot1q",
            "getval": re.compile(
                r"""
                ^\s*encapsulation\sdot1q
                \s+(?P<vlan_id>\d+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "encapsulation dot1q"
                      "{{ ' ' + dot1q|string if dot1q is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "dot1q": "{{ vlan_id }}",
                },
            },
        },
        {
            "name": "evpn_multisite_tracking",
            "getval": re.compile(
                r"""
                \s+evpn\smultisite\s
                (?P<tracking_type>dci-tracking|fabric-tracking)
                \s*$""", re.VERBOSE,
            ),
            "setval": "evpn multisite"
                      "{{ ' ' + evpn_multisite_tracking|string "
                      "if evpn_multisite_tracking is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "evpn_multisite_tracking": "{{ tracking_type }}",
                },
            },
        },
        {
            "name": "ipv4.address",
            "getval": re.compile(
                r"""
                \s+ip\saddress
                \s+(?P<address>\S+)
                (\s+(?P<ip_network_mask>(?!secondary\b)\S+))?
                (\s+(?P<secondary>secondary))?
                (\s+route-preference\s+(?P<route_preference>\d+))?
                (\s+tag\s+(?P<tag>\d+))?
                \s*$""", re.VERBOSE,
            ),
            "compval": "ipv4",
            "setval": "ip address"
            "{{ ' ' + ipv4.address.address if ipv4.address.address is defined else '' }}"
            "{{ ' ' + ipv4.address.ip_network_mask if ipv4.address.ip_network_mask is defined else '' }}"
            "{{ ' secondary' if ipv4.address.secondary | default(False) else '' }}"
            "{{ ' route-preference ' + ipv4.address.route_preference|string"
            " if ipv4.address.route_preference is defined else '' }}"
            "{{ ' tag ' + ipv4.address.tag|string if ipv4.address.tag is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "ipv4": [{
                        "address": "{{ address }}",
                        "ip_network_mask": "{{ ip_network_mask }}",
                        "route_preference": "{{ route_preference }}",
                        "tag": "{{ tag }}",
                        "secondary": "{{ not not secondary }}",
                    }],
                },
            },
        },
        {
            "name": "redirects",
            "getval": re.compile(
                r"""
                \s+ip\sredirects
                $""", re.VERBOSE,
            ),
            "setval": "ip redirects",
            "result": {
                "{{ name }}": {
                    "redirects": True,
                },
            },
        },
        {
            "name": "no_redirects",
            "getval": re.compile(
                r"""
                \s+no\sip\sredirects
                $""", re.VERBOSE,
            ),
            "setval": "no ip redirects",
            "result": {
                "{{ name }}": {
                    "redirects": False,
                },
            },
        },
        {
            "name": "unreachables",
            "getval": re.compile(
                r"""
                \s+ip\sunreachables
                $""", re.VERBOSE,
            ),
            "setval": "ip unreachables",
            "result": {
                "{{ name }}": {
                    "unreachables": True,
                },
            },
        },
        {
            "name": "proxy_arp",
            "getval": re.compile(
                r"""
                \s+ip\sproxy-arp
                $""", re.VERBOSE,
            ),
            "setval": "ip proxy-arp",
            "result": {
                "{{ name }}": {
                    "proxy_arp": True,
                },
            },
        },
        {
            "name": "port_unreachable",
            "getval": re.compile(
                r"""
                \s+ip\sport-unreachable
                $""", re.VERBOSE,
            ),
            "setval": "ip port-unreachable",
            "result": {
                "{{ name }}": {
                    "port_unreachable": True,
                },
            },
        },
        {
            "name": "verify",
            "getval": re.compile(
                r"""
                \s+ip\sverify\sunicast\ssource\sreachable-via
                \s+(?P<mode>\S+)
                (\s+(?P<allow_default>allow-default))?
                \s*$""", re.VERBOSE,
            ),
            "setval": "ip verify unicast source reachable-via"
                      "{{ ' ' + verify.unicast.source.reachable_via.mode|string "
                      "if verify.unicast.source.reachable_via.mode else ''}}"
                      "{{ ' allow-default'"
                      " if verify.unicast.source.reachable_via.allow_default|d(False) else ''}}",
            "result": {
                "{{ name }}": {
                    "verify": {
                        "unicast": {
                            "source": {
                                "reachable_via": {
                                    "mode": "{{ mode }}",
                                    "allow_default": "{{ not not allow_default }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv4",
            "getval": re.compile(
                r"""
                \s+ip\sdhcp
                \s+(?P<smart_relay>smart-relay)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ip dhcp smart-relay' if dhcp.ipv4.smart_relay|d(False) else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv4": {
                            "smart_relay": "{{ not not smart_relay }}",
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv4.option82",
            "getval": re.compile(
                r"""
                \s+ip\sdhcp\soption82\ssuboption\scircuit-id
                \s+(?P<circuit_id>\S+)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ip dhcp option82 suboption circuit-id ' + "
            "dhcp.ipv4.option82.suboption.circuit_id|string"
            " if dhcp.ipv4.option82.suboption.circuit_id is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv4": {
                            "option82": {
                                "suboption": {
                                    "circuit_id": "{{ circuit_id }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv4.information",
            "getval": re.compile(
                r"""
                \s+ip\sdhcp\srelay\sinformation
                \s+(?P<trusted>trusted)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ip dhcp relay information trusted'"
            " if dhcp.ipv4.relay.information.trusted|d(False) else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv4": {
                            "relay": {
                                "information": {
                                    "trusted": "{{ not not trusted }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv4.subnet_selection",
            "getval": re.compile(
                r"""
                \s+ip\sdhcp\srelay\ssubnet-selection
                \s+(?P<subnet_ip>\S+)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ip dhcp relay subnet-selection ' + "
            "dhcp.ipv4.relay.subnet_selection.subnet_ip|string"
            " if dhcp.ipv4.relay.subnet_selection.subnet_ip is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv4": {
                            "relay": {
                                "subnet_selection": {
                                    "subnet_ip": "{{ subnet_ip }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv4.source_interface",
            "getval": re.compile(
                r"""
                \s+ip\sdhcp\srelay\ssource-interface
                \s+(?P<interface_type>ethernet|vlan|port-channel|loopback)\s+(?P<interface_id>\S+)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ip dhcp relay source-interface ' +"
            " dhcp.ipv4.relay.source_interface.interface_type|string + ' ' +"
            " dhcp.ipv4.relay.source_interface.interface_id|string "
            "if dhcp.ipv4.relay.source_interface.interface_type is defined and "
            "dhcp.ipv4.relay.source_interface.interface_id is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv4": {
                            "relay": {
                                "source_interface": {
                                    "interface_type": "{{ interface_type }}",
                                    "interface_id": "{{ interface_id }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv4.address",
            "getval": re.compile(
                r"""\s+ip\sdhcp\srelay\saddress
                (\s+(?P<relay_ip>\S+)
                (\s+use-vrf\s+(?P<vrf_name>\S+))?)?
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ip dhcp relay address ' + dhcp.ipv4.relay_ip|string "
            "if dhcp.ipv4.relay_ip is defined else '' }}"
            "{{' use-vrf ' + dhcp.ipv4.vrf_name|string "
            "if dhcp.ipv4.vrf_name is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv4": {
                            "relay": {
                                "address": [{
                                    "relay_ip": "{{ relay_ip }}",
                                    "vrf_name": "{{ vrf_name }}",
                                }],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ipv6.address",
            "getval": re.compile(
                r"""
                \s+ipv6\saddress(\s+(?P<address>\S+)
                (\s+aggregate-prefix-length\s+(?P<aggregate_prefix_length>\S+))?
                (\s+(?P<anycast>anycast))?
                (\s+(?P<eui64>eui64))?
                (\s+route-preference\s+(?P<route_preference>\S+))?
                (\s+tag\s+(?P<tag>\S+))?
                (\s+(?P<use_bia>use-bia))?)
                \s*$""", re.VERBOSE,
            ),
            "compval": "ipv6",
            "setval": "{{ 'ipv6 address ' + ipv6.address.address|string if ipv6.address.address is defined else '' }}"
            "{{ ' aggregate-prefix-length ' + ipv6.address.aggregate_prefix_length|string if ipv6.address.aggregate_prefix_length is defined else '' }}"
            "{{ ' anycast' if ipv6.address.anycast|d(False) else '' }}"
            "{{ ' eui64' if ipv6.address.eui64|d(False) else '' }}"
            "{{ ' route-preference ' + ipv6.address.route_preference|string if ipv6.address.route_preference is defined else '' }}"
            "{{ ' tag ' + ipv6.address.tag|string if ipv6.address.tag is defined else '' }}"
            "{{ ' use-bia' if ipv6.address.use_bia|d(False) else '' }}",
            "result": {
                "{{ name }}": {
                    "ipv6": [{
                        "address": "{{ address }}",
                        "route_preference": "{{ route_preference }}",
                        "aggregate_prefix_length": "{{ aggregate_prefix_length }}",
                        "tag": "{{ tag }}",
                        "use_bia": "{{ not not use_bia }}",
                        "eui64": "{{ not not eui64 }}",
                        "anycast": "{{ not not anycast }}",
                    }],
                },
            },
        },
        {
            "name": "ipv6_redirects",
            "getval": re.compile(
                r"""
                \s+ipv6\sredirects
                $""", re.VERBOSE,
            ),
            "setval": "ipv6 redirects",
            "result": {
                "{{ name }}": {
                    "ipv6_redirects": True,
                },
            },
        },
        {
            "name": "ipv6_no_redirects",
            "getval": re.compile(
                r"""
                \s+no\sipv6\sredirects
                $""", re.VERBOSE,
            ),
            "setval": "no ipv6 redirects",
            "result": {
                "{{ name }}": {
                    "ipv6_redirects": False,
                },
            },
        },
        {
            "name": "ipv6_unreachables",
            "getval": re.compile(
                r"""
                \s+ipv6\sunreachables
                $""", re.VERBOSE,
            ),
            "setval": "ipv6 unreachables",
            "result": {
                "{{ name }}": {
                    "ipv6_unreachables": True,
                },
            },
        },
        {
            "name": "ipv6_verify",
            "getval": re.compile(
                r"""
                \s+ipv6\sverify\sunicast\ssource\sreachable-via
                \s+(?P<mode>\S+)
                (\s+(?P<allow_default>allow-default))?
                \s*$""", re.VERBOSE,
            ),
            "setval": "{{ 'ipv6 verify unicast source reachable-via ' +"
            " verify.unicast.source.reachable_via.mode|string"
            " if verify.unicast.source.reachable_via.mode is defined else '' }}"
            "{{ ' allow-default' if verify.unicast.source.reachable_via.allow_default|d(False) else '' }}",
            "result": {
                "{{ name }}": {
                    "ipv6_verify": {
                        "unicast": {
                            "source": {
                                "reachable_via": {
                                    "mode": "{{ mode }}",
                                    "allow_default": "{{ not not allow_default }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv6",
            "getval": re.compile(
                r"""
                \s+ipv6\sdhcp
                \s+(?P<smart_relay>smart-relay)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ipv6 dhcp smart-relay' if dhcp.ipv6.smart_relay|d(False) else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv6": {
                            "smart_relay": "{{ not not smart_relay }}",
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv6.source_interface",
            "getval": re.compile(
                r"""
                \s+ipv6\sdhcp\s+relay\s+source-interface
                \s+(?P<source_interface_type>ethernet|vlan|port-channel|loopback)
                \s+(?P<source_interface_id>\S+)
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ipv6 dhcp relay source-interface ' + "
            "dhcp.ipv6.source_interface.interface_type + ' ' + "
            "dhcp.ipv6.source_interface.interface_id "
            "if dhcp.ipv6.source_interface is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv6": {
                            "relay": {
                                "source_interface": {
                                    "interface_type": "{{ source_interface_type }}",
                                    "interface_id": "{{ source_interface_id }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dhcp.ipv6.address",
            "getval": re.compile(
                r"""
                ^\s*ipv6\sdhcp\srelay\saddress
                \s+(?P<relay_ip>\S+)
                (\s+interface\s+(?P<interface_type>ethernet|vlan|port-channel)\s+(?P<interface_id>\S+))?
                (\s+use-vrf\s+(?P<vrf_name>\S+))?
                \s*$""", re.VERBOSE,
            ),
            "compval": "dhcp",
            "setval": "{{ 'ipv6 dhcp relay address ' +"
            " dhcp.ipv6.relay_ip|string "
            "if dhcp.ipv6.relay_ip is defined else '' }}"
            "{{ ' interface ' + dhcp.ipv6.interface_type|string + ' ' +"
            " dhcp.ipv6.interface_id|string "
            "if dhcp.ipv6.interface_type is defined else '' }}"
            "{{ ' use-vrf ' + dhcp.ipv6.vrf_name|string "
            "if dhcp.ipv6.vrf_name is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "dhcp": {
                        "ipv6": {
                            "relay": {
                                "address": [{
                                    "relay_ip": "{{ relay_ip }}",
                                    "vrf_name": "{{ vrf_name }}",
                                    "interface_type": "{{ interface_type }}",
                                    "interface_id": "{{ interface_id }}",
                                }],
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



# Copyright (c) 2025 Markus Bergholz
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    CONNECTIVITY = r"""
options:
  username:
    description:
      - AdGuardHome user.
    required: true
    type: str
  password:
    description:
      - Related password for the AdGuardHome user.
    required: true
    type: str
  host:
    description:
      - URL of AdGuardHome host.
      - For example, V(https://my-adguard.my-domain) or V(http://192.168.1.2).
    required: true
    type: str
  validate_certs:
    description:
      - Ability to disable TLS certificate validation.
      - >
        This should only set to V(false) when the network path between the host the module is run on and
        the AdGuard host is fully under your control and trusted.
    required: false
    type: bool
    default: true
"""



#!/usr/bin/python
#
# Copyright (c) 2020 Suyeb Ansari (@suyeb786), Pallavi Chaudhari(@PallaviC2510)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = \
    '''
---
module: azure_rm_backupazurevm
version_added: '1.1.0'
short_description: Back up an Azure Virtual Machine using Azure Backup
description:
    - Back up an Azure VM using Azure Backup.
    - Enabling/Updating protection for the Azure VM.
    - Trigger an on-demand backup for a protected Azure VM.
    - Stop protection but retain existing data.
    - Stop protection and delete data.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    recovery_vault_name:
        description:
            - The name of the Azure Recovery Service Vault.
        required: true
        type: str
    resource_id:
        description:
            - Azure Virtual Machine Resource ID.
        required: true
        type: str
    backup_policy_id:
        description:
            - Backup Policy ID present under Recovery Service Vault mentioned in recovery_vault_name field.
        required: true
        type: str
    recovery_point_expiry_time:
        description:
            - Recovery Point Expiry Time in UTC.
            - This used if C(state) parameter is C(backup).
        required: false
        type: str
        version_added: '1.15.0'
    state:
        description:
            - Assert the state of the protection item.
            - Use C(create) for enabling protection for the Azure VM.
            - Use C(update) for changing the policy of protection.
            - Use C(stop) for stop protection but retain existing data.
            - Use C(delete) for stop protection and delete data.
            - Use C(backup) for on-demand backup.
        default: create
        type: str
        choices:
            - create
            - update
            - delete
            - stop
            - backup
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Suyeb Ansari (@suyeb786)
    - Pallavi Chaudhari (@PallaviC2510)

'''

EXAMPLES = \
    '''
- name: Enabling/Updating protection for the Azure VM
  azure_rm_backupazurevm:
    resource_group: 'myResourceGroup'
    recovery_vault_name: 'testVault'
    resource_id: '/subscriptions/<sub-id>/resourceGroups/my-rg/providers/Microsoft.Compute/virtualMachines/testVM'
    backup_policy_id: '/subscriptions/<sub-id>/resourceGroups/my-rg/providers/microsoft.recoveryservices/vaults/testVault/backupPolicies/ProdPolicy'
    state: 'create'
- name: Stop protection but retain existing data
  azure_rm_backupazurevm:
    resource_group: 'my-rg'
    recovery_vault_name: 'testVault'
    resource_id: '/subscriptions/<sub-id>/resourceGroups/my-rg/providers/Microsoft.Compute/virtualMachines/testVM'
    state: 'stop'
- name: Stop protection and delete data
  azure_rm_backupazurevm:
    resource_group: 'my-rg'
    recovery_vault_name: 'testVault'
    resource_id: '/subscriptions/<sub-id>/resourceGroups/my-rg/providers/Microsoft.Compute/virtualMachines/testVM'
    state: 'delete'
- name: Trigger an on-demand backup for a protected Azure VM
  azure_rm_backupazurevm:
    resource_group: 'my-rg'
    recovery_vault_name: 'testVault'
    resource_id: '/subscriptions/<sub-id>/resourceGroups/my-rg/providers/Microsoft.Compute/virtualMachines/testVM'
    backup_policy_id: '/subscriptions/<sub-id>/resourceGroups/my-rg/providers/microsoft.recoveryservices/vaults/testVault/backupPolicies/ProdPolicy'
    recovery_point_expiry_time: '2023-02-09T06:00:00Z'
    state: 'backup'
'''

RETURN = \
    '''
id:
    description:
        - VM backup protection details.
    returned: always
    type: str
    sample: '{"response":{"id":"protection_id","name":"protection_item_name","properties":{}}}'
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
import json


class Actions:
    (NoAction, Create, Update, Delete) = range(4)


class BackupAzureVM(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            recovery_vault_name=dict(
                type='str',
                required=True
            ),
            resource_id=dict(
                type='str',
                required=True
            ),
            backup_policy_id=dict(
                type='str',
                required=True
            ),
            recovery_point_expiry_time=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='create',
                choices=['create', 'update', 'delete', 'stop', 'backup']
            )
        )

        self.resource_group = None
        self.recovery_vault_name = None
        self.resource_id = None
        self.backup_policy_id = None
        self.recovery_point_expiry_time = None
        self.state = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.url = None
        self.status_code = [200, 201, 202, 204]
        self.to_do = Actions.NoAction

        self.body = {}
        self.query_parameters = {}
        self.query_parameters['api-version'] = None
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(BackupAzureVM, self).__init__(derived_arg_spec=self.module_arg_spec,
                                            supports_check_mode=True,
                                            supports_tags=True)

    def get_api_version(self):
        return '2019-05-13' if self.state == 'create' or self.state == 'update' or self.state == 'delete' or self.state == 'stop' else '2016-12-01'

    def get_url(self):
        sub_id = self.subscription_id
        if self.module.params.get('subscription_id'):
            sub_id = self.module.params.get('subscription_id')
        if self.state == 'create' or self.state == 'update' or self.state == 'delete' or self.state == 'stop':
            return '/subscriptions' + '/' + sub_id \
                   + '/resourceGroups' + '/' + self.resource_group + '/providers' \
                   + '/Microsoft.RecoveryServices' + '/vaults' + '/' \
                   + self.recovery_vault_name \
                   + '/backupFabrics/Azure/protectionContainers/' \
                   + 'iaasvmcontainer;iaasvmcontainerv2;' + self.parse_resource_to_dict(self.resource_id)['resource_group']\
                   + ';' + self.parse_resource_to_dict(self.resource_id)['name'] + '/protectedItems/' \
                   + 'vm;iaasvmcontainerv2;' + self.parse_resource_to_dict(self.resource_id)['resource_group'] + ';' \
                   + self.parse_resource_to_dict(self.resource_id)['name']
        if self.state == 'backup':
            return '/subscriptions' + '/' + sub_id \
                   + '/resourceGroups' + '/' + self.resource_group + '/providers' \
                   + '/Microsoft.RecoveryServices' + '/vaults' + '/' \
                   + self.recovery_vault_name \
                   + '/backupFabrics/Azure/protectionContainers/' \
                   + 'iaasvmcontainer;iaasvmcontainerv2;' + self.parse_resource_to_dict(self.resource_id)['resource_group'] \
                   + ';' + self.parse_resource_to_dict(self.resource_id)['name'] + '/protectedItems/' \
                   + 'vm;iaasvmcontainerv2;' + self.parse_resource_to_dict(self.resource_id)['resource_group'] + ';' \
                   + self.parse_resource_to_dict(self.resource_id)['name'] + '/backup'

    def get_body(self):
        if self.state == 'create' or self.state == 'update':
            return {
                'properties':
                    {
                        'protectedItemType': 'Microsoft.Compute/virtualMachines',
                        'sourceResourceId': self.resource_id,
                        'policyId': self.backup_policy_id
                    }
            }
        elif self.state == 'backup':
            body = {
                "properties": {
                    "objectType": "IaasVMBackupRequest"
                }
            }
            if self.recovery_point_expiry_time:
                body["properties"]["recoveryPointExpiryTimeInUTC"] = self.recovery_point_expiry_time

            return body
        elif self.state == 'stop':
            return {
                "properties": {
                    "protectedItemType": "Microsoft.Compute/virtualMachines",
                    "sourceResourceId": self.resource_id,
                    "protectionState": "ProtectionStopped"
                }
            }
        else:
            return {}

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        self.query_parameters['api-version'] = self.get_api_version()
        self.url = self.get_url()
        self.body = self.get_body()
        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        changed = False
        if self.state == 'create' or self.state == 'update':
            changed = True
            response = self.enable_update_protection_for_azure_vm()
        if self.state == 'delete':
            changed = True
            response = self.stop_protection_and_delete_data()
        if self.state == 'stop':
            changed = True
            response = self.stop_protection_but_retain_existing_data()
        if self.state == 'backup':
            changed = True
            response = self.trigger_on_demand_backup()
        self.results['response'] = response
        self.results['changed'] = changed

        return self.results

    def enable_update_protection_for_azure_vm(self):

        # self.log('Enabling/Updating protection for the Azure Virtual Machine {0}'.format(self.))

        try:
            response = self.mgmt_client.query(
                self.url,
                'PUT',
                self.query_parameters,
                self.header_parameters,
                self.body,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error in enabling/updating protection for Azure VM.')
            self.fail(
                'Error in creating/updating protection for Azure VM {0}'.format(str(e)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def stop_protection_but_retain_existing_data(self):

        # self.log('Stop protection and retain existing data{0}'.format(self.))

        try:
            response = self.mgmt_client.query(
                self.url,
                'PUT',
                self.query_parameters,
                self.header_parameters,
                self.body,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error attempting to stop protection.')
            self.fail('Error in disabling the protection: {0}'.format(str(e)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def stop_protection_and_delete_data(self):

        # self.log('Stop protection and delete data{0}'.format(self.))

        try:
            response = self.mgmt_client.query(
                self.url,
                'DELETE',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error attempting to delete backup.')
            self.fail('Error deleting the azure backup: {0}'.format(str(e)))

        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def trigger_on_demand_backup(self):

        # self.log('Trigger an on-demand backup for a protected Azure VM{0}'.format(self.))

        try:
            response = self.mgmt_client.query(
                self.url,
                'POST',
                self.query_parameters,
                self.header_parameters,
                self.body,
                self.status_code,
                600,
                30,
            )
        except Exception as e:
            self.log('Error attempting to backup azure vm.')
            self.fail(
                'Error while taking on-demand backup: {0}'.format(str(e)))
        # The return value is None, which only triggers the backup. Backups also take some time to complete.
        response = dict(msg='The backup has been successfully triggered, please monitor the backup process on the Backup Jobs page')
        return response


def main():
    BackupAzureVM()


if __name__ == '__main__':
    main()



# Copyright (c) 2020, Yevhen Khmelenko <ujenmr@gmail.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Yevhen Khmelenko (@ujenmr)
name: logstash
type: notification
short_description: Sends events to Logstash
description:
  - This callback reports facts and task events to Logstash U(https://www.elastic.co/products/logstash).
requirements:
  - whitelisting in configuration
  - logstash (Python library)
options:
  server:
    description: Address of the Logstash server.
    type: str
    env:
      - name: LOGSTASH_SERVER
    ini:
      - section: callback_logstash
        key: server
        version_added: 1.0.0
    default: localhost
  port:
    description: Port on which logstash is listening.
    type: int
    env:
      - name: LOGSTASH_PORT
    ini:
      - section: callback_logstash
        key: port
        version_added: 1.0.0
    default: 5000
  type:
    description: Message type.
    type: str
    env:
      - name: LOGSTASH_TYPE
    ini:
      - section: callback_logstash
        key: type
        version_added: 1.0.0
    default: ansible
  pre_command:
    description: Executes command before run and its result is added to the C(ansible_pre_command_output) logstash field.
    type: str
    version_added: 2.0.0
    ini:
      - section: callback_logstash
        key: pre_command
    env:
      - name: LOGSTASH_PRE_COMMAND
  format_version:
    description: Logging format.
    type: str
    version_added: 2.0.0
    ini:
      - section: callback_logstash
        key: format_version
    env:
      - name: LOGSTASH_FORMAT_VERSION
    default: v1
    choices:
      - v1
      - v2
"""

EXAMPLES = r"""
ansible.cfg: |
  # Enable Callback plugin
  [defaults]
      callback_whitelist = community.general.logstash

  [callback_logstash]
      server = logstash.example.com
      port = 5000
      pre_command = git rev-parse HEAD
      type = ansible

11-input-tcp.conf: |-
  # Enable Logstash TCP Input
  input {
          tcp {
              port => 5000
              codec => json
              add_field => { "[@metadata][beat]" => "notify" }
              add_field => { "[@metadata][type]" => "ansible" }
          }
      }
"""

import json
import logging
import os
import socket
import uuid

from ansible import context

try:
    import logstash

    HAS_LOGSTASH = True
except ImportError:
    HAS_LOGSTASH = False

from ansible.plugins.callback import CallbackBase

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


class CallbackModule(CallbackBase):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "community.general.logstash"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self):
        super().__init__()

        if not HAS_LOGSTASH:
            self.disabled = True
            self._display.warning("The required python3-logstash is not installed.")

        self.start_time = now()

    def _init_plugin(self):
        if not self.disabled:
            self.logger = logging.getLogger("python-logstash-logger")
            self.logger.setLevel(logging.DEBUG)

            self.handler = logstash.TCPLogstashHandler(
                self.ls_server, self.ls_port, version=1, message_type=self.ls_type
            )

            self.logger.addHandler(self.handler)
            self.hostname = socket.gethostname()
            self.session = str(uuid.uuid4())
            self.errors = 0

            self.base_data = {"session": self.session, "host": self.hostname}

            if self.ls_pre_command is not None:
                self.base_data["ansible_pre_command_output"] = os.popen(self.ls_pre_command).read()

            if context.CLIARGS is not None:
                self.base_data["ansible_checkmode"] = context.CLIARGS.get("check")
                self.base_data["ansible_tags"] = context.CLIARGS.get("tags")
                self.base_data["ansible_skip_tags"] = context.CLIARGS.get("skip_tags")
                self.base_data["inventory"] = context.CLIARGS.get("inventory")

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super().set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.ls_server = self.get_option("server")
        self.ls_port = int(self.get_option("port"))
        self.ls_type = self.get_option("type")
        self.ls_pre_command = self.get_option("pre_command")
        self.ls_format_version = self.get_option("format_version")

        self._init_plugin()

    def v2_playbook_on_start(self, playbook):
        data = self.base_data.copy()
        data["ansible_type"] = "start"
        data["status"] = "OK"
        data["ansible_playbook"] = playbook._file_name

        if self.ls_format_version == "v2":
            self.logger.info("START PLAYBOOK | %s", data["ansible_playbook"], extra=data)
        else:
            self.logger.info("ansible start", extra=data)

    def v2_playbook_on_stats(self, stats):
        end_time = now()
        runtime = end_time - self.start_time
        summarize_stat = {}
        for host in stats.processed.keys():
            summarize_stat[host] = stats.summarize(host)

        if self.errors == 0:
            status = "OK"
        else:
            status = "FAILED"

        data = self.base_data.copy()
        data["ansible_type"] = "finish"
        data["status"] = status
        data["ansible_playbook_duration"] = runtime.total_seconds()
        data["ansible_result"] = json.dumps(summarize_stat)  # deprecated field

        if self.ls_format_version == "v2":
            self.logger.info("FINISH PLAYBOOK | %s", json.dumps(summarize_stat), extra=data)
        else:
            self.logger.info("ansible stats", extra=data)

    def v2_playbook_on_play_start(self, play):
        self.play_id = str(play._uuid)

        if play.name:
            self.play_name = play.name

        data = self.base_data.copy()
        data["ansible_type"] = "start"
        data["status"] = "OK"
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name

        if self.ls_format_version == "v2":
            self.logger.info("START PLAY | %s", self.play_name, extra=data)
        else:
            self.logger.info("ansible play", extra=data)

    def v2_playbook_on_task_start(self, task, is_conditional):
        self.task_id = str(task._uuid)

    """
    Tasks and handler tasks are dealt with here
    """

    def v2_runner_on_ok(self, result, **kwargs):
        task_name = str(result._task).replace("TASK: ", "").replace("HANDLER: ", "")

        data = self.base_data.copy()
        if task_name == "setup":
            data["ansible_type"] = "setup"
            data["status"] = "OK"
            data["ansible_host"] = result._host.name
            data["ansible_play_id"] = self.play_id
            data["ansible_play_name"] = self.play_name
            data["ansible_task"] = task_name
            data["ansible_facts"] = self._dump_results(result._result)

            if self.ls_format_version == "v2":
                self.logger.info("SETUP FACTS | %s", self._dump_results(result._result), extra=data)
            else:
                self.logger.info("ansible facts", extra=data)
        else:
            if "changed" in result._result.keys():
                data["ansible_changed"] = result._result["changed"]
            else:
                data["ansible_changed"] = False

            data["ansible_type"] = "task"
            data["status"] = "OK"
            data["ansible_host"] = result._host.name
            data["ansible_play_id"] = self.play_id
            data["ansible_play_name"] = self.play_name
            data["ansible_task"] = task_name
            data["ansible_task_id"] = self.task_id
            data["ansible_result"] = self._dump_results(result._result)

            if self.ls_format_version == "v2":
                self.logger.info(
                    "TASK OK | %s | RESULT | %s", task_name, self._dump_results(result._result), extra=data
                )
            else:
                self.logger.info("ansible ok", extra=data)

    def v2_runner_on_skipped(self, result, **kwargs):
        task_name = str(result._task).replace("TASK: ", "").replace("HANDLER: ", "")

        data = self.base_data.copy()
        data["ansible_type"] = "task"
        data["status"] = "SKIPPED"
        data["ansible_host"] = result._host.name
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name
        data["ansible_task"] = task_name
        data["ansible_task_id"] = self.task_id
        data["ansible_result"] = self._dump_results(result._result)

        if self.ls_format_version == "v2":
            self.logger.info("TASK SKIPPED | %s", task_name, extra=data)
        else:
            self.logger.info("ansible skipped", extra=data)

    def v2_playbook_on_import_for_host(self, result, imported_file):
        data = self.base_data.copy()
        data["ansible_type"] = "import"
        data["status"] = "IMPORTED"
        data["ansible_host"] = result._host.name
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name
        data["imported_file"] = imported_file

        if self.ls_format_version == "v2":
            self.logger.info("IMPORT | %s", imported_file, extra=data)
        else:
            self.logger.info("ansible import", extra=data)

    def v2_playbook_on_not_import_for_host(self, result, missing_file):
        data = self.base_data.copy()
        data["ansible_type"] = "import"
        data["status"] = "NOT IMPORTED"
        data["ansible_host"] = result._host.name
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name
        data["imported_file"] = missing_file

        if self.ls_format_version == "v2":
            self.logger.info("NOT IMPORTED | %s", missing_file, extra=data)
        else:
            self.logger.info("ansible import", extra=data)

    def v2_runner_on_failed(self, result, **kwargs):
        task_name = str(result._task).replace("TASK: ", "").replace("HANDLER: ", "")

        data = self.base_data.copy()
        if "changed" in result._result.keys():
            data["ansible_changed"] = result._result["changed"]
        else:
            data["ansible_changed"] = False

        data["ansible_type"] = "task"
        data["status"] = "FAILED"
        data["ansible_host"] = result._host.name
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name
        data["ansible_task"] = task_name
        data["ansible_task_id"] = self.task_id
        data["ansible_result"] = self._dump_results(result._result)

        self.errors += 1
        if self.ls_format_version == "v2":
            self.logger.error(
                "TASK FAILED | %s | HOST | %s | RESULT | %s",
                task_name,
                self.hostname,
                self._dump_results(result._result),
                extra=data,
            )
        else:
            self.logger.error("ansible failed", extra=data)

    def v2_runner_on_unreachable(self, result, **kwargs):
        task_name = str(result._task).replace("TASK: ", "").replace("HANDLER: ", "")

        data = self.base_data.copy()
        data["ansible_type"] = "task"
        data["status"] = "UNREACHABLE"
        data["ansible_host"] = result._host.name
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name
        data["ansible_task"] = task_name
        data["ansible_task_id"] = self.task_id
        data["ansible_result"] = self._dump_results(result._result)

        self.errors += 1
        if self.ls_format_version == "v2":
            self.logger.error(
                "UNREACHABLE | %s | HOST | %s | RESULT | %s",
                task_name,
                self.hostname,
                self._dump_results(result._result),
                extra=data,
            )
        else:
            self.logger.error("ansible unreachable", extra=data)

    def v2_runner_on_async_failed(self, result, **kwargs):
        task_name = str(result._task).replace("TASK: ", "").replace("HANDLER: ", "")

        data = self.base_data.copy()
        data["ansible_type"] = "task"
        data["status"] = "FAILED"
        data["ansible_host"] = result._host.name
        data["ansible_play_id"] = self.play_id
        data["ansible_play_name"] = self.play_name
        data["ansible_task"] = task_name
        data["ansible_task_id"] = self.task_id
        data["ansible_result"] = self._dump_results(result._result)

        self.errors += 1
        if self.ls_format_version == "v2":
            self.logger.error(
                "ASYNC FAILED | %s | HOST | %s | RESULT | %s",
                task_name,
                self.hostname,
                self._dump_results(result._result),
                extra=data,
            )
        else:
            self.logger.error("ansible async", extra=data)
